@@ -25,10 +25,10 @@
  * using intel's AES instruction set. 
  */
 
-#include <gnutls_errors.h>
-#include <gnutls_int.h>
+#include "errors.h"
+#include "gnutls_int.h"
 #include <gnutls/crypto.h>
-#include <gnutls_errors.h>
+#include "errors.h"
 #include <aes-x86.h>
 #include <sha-x86.h>
 #include <x86-common.h>
@@ -41,10 +41,12 @@
 /* ebx, ecx, edx 
  * This is a format compatible with openssl's CPUID detection.
  */
-#ifdef __GNUC__
+#if defined(__GNUC__)
 __attribute__((visibility("hidden")))
+#elif defined(__SUNPRO_C)
+__hidden
 #endif
-unsigned int _gnutls_x86_cpuid_s[3];
+unsigned int _gnutls_x86_cpuid_s[4];
 
 #ifndef bit_PCLMUL
 # define bit_PCLMUL 0x2
@@ -58,6 +60,14 @@ unsigned int _gnutls_x86_cpuid_s[3];
 # define bit_AES 0x2000000
 #endif
 
+#ifndef bit_AVX
+# define bit_AVX 0x10000000
+#endif
+
+#ifndef bit_MOVBE
+# define bit_MOVBE 0x00400000
+#endif
+
 #define via_bit_PADLOCK (0x3 << 6)
 #define via_bit_PADLOCK_PHE (0x3 << 10)
 #define via_bit_PADLOCK_PHE_SHA512 (0x3 << 25)
@@ -68,6 +78,7 @@ unsigned int _gnutls_x86_cpuid_s[3];
 #define INTEL_AES_NI (1<<1)
 #define INTEL_SSSE3 (1<<2)
 #define INTEL_PCLMUL (1<<3)
+#define INTEL_AVX (1<<4)
 #define VIA_PADLOCK (1<<20)
 #define VIA_PADLOCK_PHE (1<<21)
 #define VIA_PADLOCK_PHE_SHA512 (1<<22)
@@ -101,6 +112,16 @@ static void capabilities_to_intel_cpuid(unsigned capabilities)
 			    ("SSSE3 acceleration requested but not available\n");
 		}
 	}
+
+	if (capabilities & INTEL_AVX) {
+		if ((b & bit_AVX) && (b & bit_MOVBE)) {
+			_gnutls_x86_cpuid_s[1] |= bit_AVX|bit_MOVBE;
+		} else {
+			_gnutls_debug_log
+			    ("AVX acceleration requested but not available\n");
+		}
+	}
+
 	if (capabilities & INTEL_PCLMUL) {
 		if (b & bit_PCLMUL) {
 			_gnutls_x86_cpuid_s[1] |= bit_PCLMUL;
@@ -109,6 +130,7 @@ static void capabilities_to_intel_cpuid(unsigned capabilities)
 			    ("PCLMUL acceleration requested but not available\n");
 		}
 	}
+
 }
 
 static unsigned check_optimized_aes(void)
@@ -122,6 +144,11 @@ static unsigned check_ssse3(void)
 }
 
 #ifdef ASM_X86_64
+static unsigned check_avx_movbe(void)
+{
+	return ((_gnutls_x86_cpuid_s[1] & bit_AVX) && (_gnutls_x86_cpuid_s[1] & bit_MOVBE));
+}
+
 static unsigned check_pclmul(void)
 {
 	return (_gnutls_x86_cpuid_s[1] & bit_PCLMUL);
@@ -237,7 +264,7 @@ void register_x86_padlock_crypto(unsigned capabilities)
 		    ("Padlock AES accelerator was detected\n");
 		ret =
 		    gnutls_crypto_single_cipher_register
-		    (GNUTLS_CIPHER_AES_128_CBC, 80, &_gnutls_aes_padlock);
+		    (GNUTLS_CIPHER_AES_128_CBC, 80, &_gnutls_aes_padlock, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -246,21 +273,21 @@ void register_x86_padlock_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_cipher_register
 		    (GNUTLS_CIPHER_AES_128_GCM, 80,
-		     &_gnutls_aes_gcm_padlock);
+		     &_gnutls_aes_gcm_padlock, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
 #ifdef HAVE_LIBNETTLE
 		ret =
 		    gnutls_crypto_single_cipher_register
-		    (GNUTLS_CIPHER_AES_192_CBC, 80, &_gnutls_aes_padlock);
+		    (GNUTLS_CIPHER_AES_192_CBC, 80, &_gnutls_aes_padlock, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
 
 		ret =
 		    gnutls_crypto_single_cipher_register
-		    (GNUTLS_CIPHER_AES_256_CBC, 80, &_gnutls_aes_padlock);
+		    (GNUTLS_CIPHER_AES_256_CBC, 80, &_gnutls_aes_padlock, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -268,12 +295,15 @@ void register_x86_padlock_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_cipher_register
 		    (GNUTLS_CIPHER_AES_256_GCM, 80,
-		     &_gnutls_aes_gcm_padlock);
+		     &_gnutls_aes_gcm_padlock, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
 #endif
+	} else {
+		_gnutls_priority_update_non_aesni();
 	}
+
 #ifdef HAVE_LIBNETTLE
 	phe = check_phe(edx);
 
@@ -286,7 +316,7 @@ void register_x86_padlock_crypto(unsigned capabilities)
 			ret =
 			    gnutls_crypto_single_digest_register
 			    (GNUTLS_DIG_SHA384, 80,
-			     &_gnutls_sha_padlock_nano);
+			     &_gnutls_sha_padlock_nano, 0);
 			if (ret < 0) {
 				gnutls_assert();
 			}
@@ -294,7 +324,7 @@ void register_x86_padlock_crypto(unsigned capabilities)
 			ret =
 			    gnutls_crypto_single_digest_register
 			    (GNUTLS_DIG_SHA512, 80,
-			     &_gnutls_sha_padlock_nano);
+			     &_gnutls_sha_padlock_nano, 0);
 			if (ret < 0) {
 				gnutls_assert();
 			}
@@ -302,7 +332,7 @@ void register_x86_padlock_crypto(unsigned capabilities)
 			ret =
 			    gnutls_crypto_single_mac_register
 			    (GNUTLS_MAC_SHA384, 80,
-			     &_gnutls_hmac_sha_padlock_nano);
+			     &_gnutls_hmac_sha_padlock_nano, 0);
 			if (ret < 0) {
 				gnutls_assert();
 			}
@@ -310,7 +340,7 @@ void register_x86_padlock_crypto(unsigned capabilities)
 			ret =
 			    gnutls_crypto_single_mac_register
 			    (GNUTLS_MAC_SHA512, 80,
-			     &_gnutls_hmac_sha_padlock_nano);
+			     &_gnutls_hmac_sha_padlock_nano, 0);
 			if (ret < 0) {
 				gnutls_assert();
 			}
@@ -319,7 +349,7 @@ void register_x86_padlock_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_digest_register(GNUTLS_DIG_SHA1,
 							 80,
-							 &_gnutls_sha_padlock_nano);
+							 &_gnutls_sha_padlock_nano, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -327,7 +357,7 @@ void register_x86_padlock_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_digest_register(GNUTLS_DIG_SHA224,
 							 80,
-							 &_gnutls_sha_padlock_nano);
+							 &_gnutls_sha_padlock_nano, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -335,7 +365,7 @@ void register_x86_padlock_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_digest_register(GNUTLS_DIG_SHA256,
 							 80,
-							 &_gnutls_sha_padlock_nano);
+							 &_gnutls_sha_padlock_nano, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -343,7 +373,7 @@ void register_x86_padlock_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_mac_register(GNUTLS_MAC_SHA1,
 						      80,
-						      &_gnutls_hmac_sha_padlock_nano);
+						      &_gnutls_hmac_sha_padlock_nano, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -353,7 +383,7 @@ void register_x86_padlock_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_mac_register(GNUTLS_MAC_SHA256,
 						      80,
-						      &_gnutls_hmac_sha_padlock_nano);
+						      &_gnutls_hmac_sha_padlock_nano, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -365,7 +395,7 @@ void register_x86_padlock_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_digest_register(GNUTLS_DIG_SHA1,
 							 80,
-							 &_gnutls_sha_padlock);
+							 &_gnutls_sha_padlock, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -373,7 +403,7 @@ void register_x86_padlock_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_digest_register(GNUTLS_DIG_SHA256,
 							 80,
-							 &_gnutls_sha_padlock);
+							 &_gnutls_sha_padlock, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -381,7 +411,7 @@ void register_x86_padlock_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_mac_register(GNUTLS_MAC_SHA1,
 						      80,
-						      &_gnutls_hmac_sha_padlock);
+						      &_gnutls_hmac_sha_padlock, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -389,7 +419,7 @@ void register_x86_padlock_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_mac_register(GNUTLS_MAC_SHA256,
 						      80,
-						      &_gnutls_hmac_sha_padlock);
+						      &_gnutls_hmac_sha_padlock, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -438,7 +468,7 @@ void register_x86_intel_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_cipher_register
 		    (GNUTLS_CIPHER_AES_128_GCM, 90,
-		     &_gnutls_aes_gcm_x86_ssse3);
+		     &_gnutls_aes_gcm_x86_ssse3, 0);
 			if (ret < 0) {
 				gnutls_assert();
 			}
@@ -446,28 +476,28 @@ void register_x86_intel_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_cipher_register
 		    (GNUTLS_CIPHER_AES_256_GCM, 90,
-		     &_gnutls_aes_gcm_x86_ssse3);
+		     &_gnutls_aes_gcm_x86_ssse3, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
 
 		ret =
 		    gnutls_crypto_single_cipher_register
-		    (GNUTLS_CIPHER_AES_128_CBC, 90, &_gnutls_aes_ssse3);
+		    (GNUTLS_CIPHER_AES_128_CBC, 90, &_gnutls_aes_ssse3, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
 
 		ret =
 		    gnutls_crypto_single_cipher_register
-		    (GNUTLS_CIPHER_AES_192_CBC, 90, &_gnutls_aes_ssse3);
+		    (GNUTLS_CIPHER_AES_192_CBC, 90, &_gnutls_aes_ssse3, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
 
 		ret =
 		    gnutls_crypto_single_cipher_register
-		    (GNUTLS_CIPHER_AES_256_CBC, 90, &_gnutls_aes_ssse3);
+		    (GNUTLS_CIPHER_AES_256_CBC, 90, &_gnutls_aes_ssse3, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -475,7 +505,7 @@ void register_x86_intel_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_digest_register(GNUTLS_DIG_SHA1,
 							 80,
-							 &_gnutls_sha_x86_ssse3);
+							 &_gnutls_sha_x86_ssse3, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -483,7 +513,7 @@ void register_x86_intel_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_digest_register(GNUTLS_DIG_SHA224,
 							 80,
-							 &_gnutls_sha_x86_ssse3);
+							 &_gnutls_sha_x86_ssse3, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -491,7 +521,7 @@ void register_x86_intel_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_digest_register(GNUTLS_DIG_SHA256,
 							 80,
-							 &_gnutls_sha_x86_ssse3);
+							 &_gnutls_sha_x86_ssse3, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
@@ -500,21 +530,21 @@ void register_x86_intel_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_mac_register(GNUTLS_MAC_SHA1,
 							 80,
-							 &_gnutls_hmac_sha_x86_ssse3);
+							 &_gnutls_hmac_sha_x86_ssse3, 0);
 		if (ret < 0)
 			gnutls_assert();
 
 		ret =
 		    gnutls_crypto_single_mac_register(GNUTLS_MAC_SHA224,
 							 80,
-							 &_gnutls_hmac_sha_x86_ssse3);
+							 &_gnutls_hmac_sha_x86_ssse3, 0);
 		if (ret < 0)
 			gnutls_assert();
 
 		ret =
 		    gnutls_crypto_single_mac_register(GNUTLS_MAC_SHA256,
 							 80,
-							 &_gnutls_hmac_sha_x86_ssse3);
+							 &_gnutls_hmac_sha_x86_ssse3, 0);
 		if (ret < 0)
 			gnutls_assert();
 
@@ -522,27 +552,27 @@ void register_x86_intel_crypto(unsigned capabilities)
 		ret =
 		    gnutls_crypto_single_digest_register(GNUTLS_DIG_SHA384,
 							 80,
-							 &_gnutls_sha_x86_ssse3);
+							 &_gnutls_sha_x86_ssse3, 0);
 		if (ret < 0)
 			gnutls_assert();
 
 		ret =
 		    gnutls_crypto_single_digest_register(GNUTLS_DIG_SHA512,
 							 80,
-							 &_gnutls_sha_x86_ssse3);
+							 &_gnutls_sha_x86_ssse3, 0);
 		if (ret < 0)
 			gnutls_assert();
 		ret =
 		    gnutls_crypto_single_mac_register(GNUTLS_MAC_SHA384,
 							 80,
-							 &_gnutls_hmac_sha_x86_ssse3);
+							 &_gnutls_hmac_sha_x86_ssse3, 0);
 		if (ret < 0)
 			gnutls_assert();
 
 		ret =
 		    gnutls_crypto_single_mac_register(GNUTLS_MAC_SHA512,
 							 80,
-							 &_gnutls_hmac_sha_x86_ssse3);
+							 &_gnutls_hmac_sha_x86_ssse3, 0);
 		if (ret < 0)
 			gnutls_assert();
 #endif
@@ -552,43 +582,96 @@ void register_x86_intel_crypto(unsigned capabilities)
 		_gnutls_debug_log("Intel AES accelerator was detected\n");
 		ret =
 		    gnutls_crypto_single_cipher_register
-		    (GNUTLS_CIPHER_AES_128_CBC, 80, &_gnutls_aesni_x86);
+		    (GNUTLS_CIPHER_AES_128_CBC, 80, &_gnutls_aesni_x86, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
 
 		ret =
 		    gnutls_crypto_single_cipher_register
-		    (GNUTLS_CIPHER_AES_192_CBC, 80, &_gnutls_aesni_x86);
+		    (GNUTLS_CIPHER_AES_192_CBC, 80, &_gnutls_aesni_x86, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
 
 		ret =
 		    gnutls_crypto_single_cipher_register
-		    (GNUTLS_CIPHER_AES_256_CBC, 80, &_gnutls_aesni_x86);
+		    (GNUTLS_CIPHER_AES_256_CBC, 80, &_gnutls_aesni_x86, 0);
 		if (ret < 0) {
 			gnutls_assert();
 		}
+
+		ret =
+		    gnutls_crypto_single_cipher_register
+		    (GNUTLS_CIPHER_AES_128_CCM, 80,
+		     &_gnutls_aes_ccm_x86_aesni, 0);
+		if (ret < 0) {
+			gnutls_assert();
+		}
+
+		ret =
+		    gnutls_crypto_single_cipher_register
+		    (GNUTLS_CIPHER_AES_256_CCM, 80,
+		     &_gnutls_aes_ccm_x86_aesni, 0);
+		if (ret < 0) {
+			gnutls_assert();
+		}
+
+		ret =
+		    gnutls_crypto_single_cipher_register
+		    (GNUTLS_CIPHER_AES_128_CCM_8, 80,
+		     &_gnutls_aes_ccm_x86_aesni, 0);
+		if (ret < 0) {
+			gnutls_assert();
+		}
+
+		ret =
+		    gnutls_crypto_single_cipher_register
+		    (GNUTLS_CIPHER_AES_256_CCM_8, 80,
+		     &_gnutls_aes_ccm_x86_aesni, 0);
+		if (ret < 0) {
+			gnutls_assert();
+		}
+
 #ifdef ASM_X86_64
 		if (check_pclmul()) {
 			/* register GCM ciphers */
-			_gnutls_debug_log
-			    ("Intel GCM accelerator was detected\n");
-			ret =
-			    gnutls_crypto_single_cipher_register
-			    (GNUTLS_CIPHER_AES_128_GCM, 80,
-			     &_gnutls_aes_gcm_pclmul);
-			if (ret < 0) {
-				gnutls_assert();
-			}
+			if (check_avx_movbe()) {
+				_gnutls_debug_log
+				    ("Intel GCM accelerator (AVX) was detected\n");
+				ret =
+				    gnutls_crypto_single_cipher_register
+				    (GNUTLS_CIPHER_AES_128_GCM, 80,
+				     &_gnutls_aes_gcm_pclmul_avx, 0);
+				if (ret < 0) {
+					gnutls_assert();
+				}
 
-			ret =
-			    gnutls_crypto_single_cipher_register
-			    (GNUTLS_CIPHER_AES_256_GCM, 80,
-			     &_gnutls_aes_gcm_pclmul);
-			if (ret < 0) {
-				gnutls_assert();
+				ret =
+				    gnutls_crypto_single_cipher_register
+				    (GNUTLS_CIPHER_AES_256_GCM, 80,
+				     &_gnutls_aes_gcm_pclmul_avx, 0);
+				if (ret < 0) {
+					gnutls_assert();
+				}
+			} else {
+				_gnutls_debug_log
+				    ("Intel GCM accelerator was detected\n");
+				ret =
+				    gnutls_crypto_single_cipher_register
+				    (GNUTLS_CIPHER_AES_128_GCM, 80,
+				     &_gnutls_aes_gcm_pclmul, 0);
+				if (ret < 0) {
+					gnutls_assert();
+				}
+
+				ret =
+				    gnutls_crypto_single_cipher_register
+				    (GNUTLS_CIPHER_AES_256_GCM, 80,
+				     &_gnutls_aes_gcm_pclmul, 0);
+				if (ret < 0) {
+					gnutls_assert();
+				}
 			}
 		} else
 #endif
@@ -596,7 +679,7 @@ void register_x86_intel_crypto(unsigned capabilities)
 			ret =
 			    gnutls_crypto_single_cipher_register
 			    (GNUTLS_CIPHER_AES_128_GCM, 80,
-			     &_gnutls_aes_gcm_x86_aesni);
+			     &_gnutls_aes_gcm_x86_aesni, 0);
 			if (ret < 0) {
 				gnutls_assert();
 			}
@@ -604,13 +687,13 @@ void register_x86_intel_crypto(unsigned capabilities)
 			ret =
 			    gnutls_crypto_single_cipher_register
 			    (GNUTLS_CIPHER_AES_256_GCM, 80,
-			     &_gnutls_aes_gcm_x86_aesni);
+			     &_gnutls_aes_gcm_x86_aesni, 0);
 			if (ret < 0) {
 				gnutls_assert();
 			}
-
-
 		}
+	} else {
+		_gnutls_priority_update_non_aesni();
 	}
 
 	return;
@@ -625,7 +708,7 @@ void register_x86_crypto(void)
 	if (p) {
 		capabilities = strtol(p, NULL, 0);
 	}
-	
+
 	register_x86_intel_crypto(capabilities);
 #ifdef ENABLE_PADLOCK
 	register_x86_padlock_crypto(capabilities);
