@@ -16,7 +16,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -25,7 +25,7 @@
 #ifdef ENABLE_ANON
 
 #include "errors.h"
-#include "auth/anon.h"
+#include <auth/anon.h>
 #include "auth.h"
 #include "dh.h"
 #include "num.h"
@@ -37,7 +37,8 @@
  *
  * Free a gnutls_anon_server_credentials_t structure.
  **/
-void gnutls_anon_free_server_credentials(gnutls_anon_server_credentials_t sc)
+void
+gnutls_anon_free_server_credentials(gnutls_anon_server_credentials_t sc)
 {
 	if (sc->deinit_dh_params) {
 		gnutls_dh_params_deinit(sc->dh_params);
@@ -53,27 +54,31 @@ void gnutls_anon_free_server_credentials(gnutls_anon_server_credentials_t sc)
  *
  * Returns: %GNUTLS_E_SUCCESS on success, or an error code.
  **/
-int gnutls_anon_allocate_server_credentials(gnutls_anon_server_credentials_t *sc)
+int
+gnutls_anon_allocate_server_credentials(gnutls_anon_server_credentials_t *
+					sc)
 {
+
 	*sc = gnutls_calloc(1, sizeof(anon_server_credentials_st));
-	if (*sc == NULL)
-		return GNUTLS_E_MEMORY_ERROR;
 
 	return 0;
 }
 
+
 /**
  * gnutls_anon_free_client_credentials:
  * @sc: is a #gnutls_anon_client_credentials_t type.
  *
  * Free a gnutls_anon_client_credentials_t structure.
  **/
-void gnutls_anon_free_client_credentials(gnutls_anon_client_credentials_t sc)
+void
+gnutls_anon_free_client_credentials(gnutls_anon_client_credentials_t sc)
 {
 }
 
 static struct gnutls_anon_client_credentials_st anon_dummy_struct;
-static const gnutls_anon_client_credentials_t anon_dummy = &anon_dummy_struct;
+static const gnutls_anon_client_credentials_t anon_dummy =
+    &anon_dummy_struct;
 
 /**
  * gnutls_anon_allocate_client_credentials:
@@ -83,7 +88,9 @@ static const gnutls_anon_client_credentials_t anon_dummy = &anon_dummy_struct;
  *
  * Returns: %GNUTLS_E_SUCCESS on success, or an error code.
  **/
-int gnutls_anon_allocate_client_credentials(gnutls_anon_client_credentials_t *sc)
+int
+gnutls_anon_allocate_client_credentials(gnutls_anon_client_credentials_t *
+					sc)
 {
 	/* anon_dummy is only there for *sc not to be null.
 	 * it is not used at all;
@@ -101,13 +108,10 @@ int gnutls_anon_allocate_client_credentials(gnutls_anon_client_credentials_t *sc
  * This function will set the Diffie-Hellman parameters for an
  * anonymous server to use.  These parameters will be used in
  * Anonymous Diffie-Hellman cipher suites.
- *
- * Deprecated: This function is unnecessary and discouraged on GnuTLS 3.6.0
- * or later. Since 3.6.0, DH parameters are negotiated
- * following RFC7919.
  **/
-void gnutls_anon_set_server_dh_params(gnutls_anon_server_credentials_t res,
-				      gnutls_dh_params_t dh_params)
+void
+gnutls_anon_set_server_dh_params(gnutls_anon_server_credentials_t res,
+				 gnutls_dh_params_t dh_params)
 {
 	if (res->deinit_dh_params) {
 		res->deinit_dh_params = 0;
@@ -116,33 +120,40 @@ void gnutls_anon_set_server_dh_params(gnutls_anon_server_credentials_t res,
 	}
 
 	res->dh_params = dh_params;
-	res->dh_sec_param = gnutls_pk_bits_to_sec_param(
-		GNUTLS_PK_DH, _gnutls_mpi_get_nbits(dh_params->params[0]));
 }
 
 /**
  * gnutls_anon_set_server_known_dh_params:
  * @res: is a gnutls_anon_server_credentials_t type
- * @sec_param: is an option of the %gnutls_sec_param_t enumeration
+ * @dh_params: The Diffie-Hellman parameters.
  *
  * This function will set the Diffie-Hellman parameters for an
  * anonymous server to use.  These parameters will be used in
  * Anonymous Diffie-Hellman cipher suites and will be selected from
  * the FFDHE set of RFC7919 according to the security level provided.
  *
- * Deprecated: This function is unnecessary and discouraged on GnuTLS 3.6.0
- * or later. Since 3.6.0, DH parameters are negotiated
- * following RFC7919.
- *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
  *   negative error value.
  *
  * Since: 3.5.6
  **/
-int gnutls_anon_set_server_known_dh_params(gnutls_anon_server_credentials_t res,
-					   gnutls_sec_param_t sec_param)
+int
+gnutls_anon_set_server_known_dh_params(gnutls_anon_server_credentials_t res,
+					gnutls_sec_param_t sec_param)
 {
-	res->dh_sec_param = sec_param;
+	int ret;
+
+	if (res->deinit_dh_params) {
+		res->deinit_dh_params = 0;
+		gnutls_dh_params_deinit(res->dh_params);
+		res->dh_params = NULL;
+	}
+
+	ret = _gnutls_set_cred_dh_params(&res->dh_params, sec_param);
+	if (ret < 0)
+		return gnutls_assert_val(ret);
+
+	res->deinit_dh_params = 1;
 
 	return 0;
 }
@@ -155,14 +166,10 @@ int gnutls_anon_set_server_known_dh_params(gnutls_anon_server_credentials_t res,
  * This function will set a callback in order for the server to get
  * the Diffie-Hellman parameters for anonymous authentication.  The
  * callback should return %GNUTLS_E_SUCCESS (0) on success.
- *
- * Deprecated: This function is unnecessary and discouraged on GnuTLS 3.6.0
- * or later. Since 3.6.0, DH parameters are negotiated
- * following RFC7919.
- *
  **/
-void gnutls_anon_set_server_params_function(
-	gnutls_anon_server_credentials_t res, gnutls_params_function *func)
+void
+gnutls_anon_set_server_params_function(gnutls_anon_server_credentials_t
+				       res, gnutls_params_function * func)
 {
 	res->params_func = func;
 }
@@ -175,14 +182,10 @@ void gnutls_anon_set_server_params_function(
  * This function will set a callback in order for the server to get
  * the Diffie-Hellman or RSA parameters for anonymous authentication.
  * The callback should return %GNUTLS_E_SUCCESS (0) on success.
- *
- * Deprecated: This function is unnecessary and discouraged on GnuTLS 3.6.0
- * or later. Since 3.6.0, DH parameters are negotiated
- * following RFC7919.
- *
  **/
-void gnutls_anon_set_params_function(gnutls_anon_server_credentials_t res,
-				     gnutls_params_function *func)
+void
+gnutls_anon_set_params_function(gnutls_anon_server_credentials_t res,
+				gnutls_params_function * func)
 {
 	res->params_func = func;
 }
