@@ -16,7 +16,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -29,8 +29,6 @@
 #include <db.h>
 #include <session_pack.h>
 #include <datum.h>
-#include "ext/server_name.h"
-#include <intprops.h>
 
 /**
  * gnutls_db_set_retrieve_function:
@@ -48,8 +46,9 @@
  * The first argument to @retr_func will be null unless
  * gnutls_db_set_ptr() has been called.
  **/
-void gnutls_db_set_retrieve_function(gnutls_session_t session,
-				     gnutls_db_retr_func retr_func)
+void
+gnutls_db_set_retrieve_function(gnutls_session_t session,
+				gnutls_db_retr_func retr_func)
 {
 	session->internals.db_retrieve_func = retr_func;
 }
@@ -65,8 +64,9 @@ void gnutls_db_set_retrieve_function(gnutls_session_t session,
  * The first argument to @rem_func will be null unless
  * gnutls_db_set_ptr() has been called.
  **/
-void gnutls_db_set_remove_function(gnutls_session_t session,
-				   gnutls_db_remove_func rem_func)
+void
+gnutls_db_set_remove_function(gnutls_session_t session,
+			      gnutls_db_remove_func rem_func)
 {
 	session->internals.db_remove_func = rem_func;
 }
@@ -82,8 +82,9 @@ void gnutls_db_set_remove_function(gnutls_session_t session,
  * The first argument to @store_func will be null unless
  * gnutls_db_set_ptr() has been called.
  **/
-void gnutls_db_set_store_function(gnutls_session_t session,
-				  gnutls_db_store_func store_func)
+void
+gnutls_db_set_store_function(gnutls_session_t session,
+			     gnutls_db_store_func store_func)
 {
 	session->internals.db_store_func = store_func;
 }
@@ -120,18 +121,12 @@ void *gnutls_db_get_ptr(gnutls_session_t session)
  * @session: is a #gnutls_session_t type.
  * @seconds: is the number of seconds.
  *
- * Set the expiration time for resumed sessions. The default is 21600
- * (6 hours) at the time of writing.
- *
- * The maximum value that can be set using this function is 604800
- * (7 days).
- *
+ * Set the expiration time for resumed sessions. The default is 3600
+ * (one hour) at the time of this writing.
  **/
 void gnutls_db_set_cache_expiration(gnutls_session_t session, int seconds)
 {
 	session->internals.expire_time = seconds;
-	if (session->internals.expire_time > 604800)
-		session->internals.expire_time = 604800;
 }
 
 /**
@@ -153,11 +148,10 @@ unsigned gnutls_db_get_default_cache_expiration(void)
  *
  * Returns: Returns %GNUTLS_E_EXPIRED, if the database entry has
  *   expired or 0 otherwise.
- *
- * Deprecated: This function is deprecated.
  **/
-int gnutls_db_check_entry(gnutls_session_t session,
-			  gnutls_datum_t session_entry)
+int
+gnutls_db_check_entry(gnutls_session_t session,
+		      gnutls_datum_t session_entry)
 {
 	return 0;
 }
@@ -165,13 +159,14 @@ int gnutls_db_check_entry(gnutls_session_t session,
 /**
  * gnutls_db_check_entry_time:
  * @entry: is a pointer to a #gnutls_datum_t type.
+ * @t: is the time of the session handshake
  *
  * This function returns the time that this entry was active.
  * It can be used for database entry expiration.
  *
  * Returns: The time this entry was created, or zero on error.
  **/
-time_t gnutls_db_check_entry_time(gnutls_datum_t *entry)
+time_t gnutls_db_check_entry_time(gnutls_datum_t * entry)
 {
 	uint32_t t;
 	uint32_t magic;
@@ -189,40 +184,6 @@ time_t gnutls_db_check_entry_time(gnutls_datum_t *entry)
 	return t;
 }
 
-/**
- * gnutls_db_check_entry_expire_time:
- * @entry: is a pointer to a #gnutls_datum_t type.
- *
- * This function returns the time that this entry will expire.
- * It can be used for database entry expiration.
- *
- * Returns: The time this entry will expire, or zero on error.
- *
- * Since: 3.6.5
- **/
-time_t gnutls_db_check_entry_expire_time(gnutls_datum_t *entry)
-{
-	uint32_t t;
-	uint32_t e;
-	uint32_t magic;
-
-	if (entry->size < 12)
-		return gnutls_assert_val(0);
-
-	magic = _gnutls_read_uint32(entry->data);
-
-	if (magic != PACKED_SESSION_MAGIC)
-		return gnutls_assert_val(0);
-
-	t = _gnutls_read_uint32(&entry->data[4]);
-	e = _gnutls_read_uint32(&entry->data[8]);
-
-	if (INT_ADD_OVERFLOW(t, e))
-		return gnutls_assert_val(0);
-
-	return t + e;
-}
-
 /* Checks if both db_store and db_retrieve functions have
  * been set up.
  */
@@ -237,8 +198,9 @@ static int db_func_is_ok(gnutls_session_t session)
 
 /* Stores session data to the db backend.
  */
-static int store_session(gnutls_session_t session, gnutls_datum_t session_id,
-			 gnutls_datum_t session_data)
+static int
+store_session(gnutls_session_t session,
+	      gnutls_datum_t session_id, gnutls_datum_t session_data)
 {
 	int ret = 0;
 
@@ -267,7 +229,7 @@ int _gnutls_server_register_current_session(gnutls_session_t session)
 	key.data = session->security_parameters.session_id;
 	key.size = session->security_parameters.session_id_size;
 
-	if (!session->internals.resumable) {
+	if (session->internals.resumable == RESUME_FALSE) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_SESSION;
 	}
@@ -291,34 +253,16 @@ int _gnutls_server_register_current_session(gnutls_session_t session)
 
 int _gnutls_check_resumed_params(gnutls_session_t session)
 {
-	time_t timestamp = gnutls_time(0);
-	const version_entry_st *vers;
-
-	/* check whether the session is expired */
-	if (timestamp - session->internals.resumed_security_parameters.timestamp >
-		    session->internals.expire_time ||
-	    session->internals.resumed_security_parameters.timestamp >
-		    timestamp)
-		return gnutls_assert_val(GNUTLS_E_EXPIRED);
-
-	/* check various parameters applicable to resumption in TLS1.2 or earlier
-	 */
-	vers = get_version(session);
-	if (!vers || !vers->tls13_sem) {
-		if (session->internals.resumed_security_parameters
-			    .ext_master_secret !=
-		    session->security_parameters.ext_master_secret)
-			return gnutls_assert_val(GNUTLS_E_INVALID_SESSION);
-
-		if (!_gnutls_server_name_matches_resumed(session))
-			return gnutls_assert_val(GNUTLS_E_INVALID_SESSION);
-	}
+	if (session->internals.resumed_security_parameters.ext_master_secret != 
+	    session->security_parameters.ext_master_secret)
+	    return gnutls_assert_val(GNUTLS_E_INVALID_SESSION);
 
 	return 0;
 }
 
-int _gnutls_server_restore_session(gnutls_session_t session,
-				   uint8_t *session_id, int session_id_size)
+int
+_gnutls_server_restore_session(gnutls_session_t session,
+			       uint8_t * session_id, int session_id_size)
 {
 	gnutls_datum_t data;
 	gnutls_datum_t key;
@@ -329,15 +273,14 @@ int _gnutls_server_restore_session(gnutls_session_t session,
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
-	if (session->internals.premaster_set !=
-	    0) { /* hack for CISCO's DTLS-0.9 */
+	if (session->internals.premaster_set != 0) {	/* hack for CISCO's DTLS-0.9 */
 		if (session_id_size ==
-			    session->internals.resumed_security_parameters
-				    .session_id_size &&
-		    memcmp(session_id,
-			   session->internals.resumed_security_parameters
-				   .session_id,
-			   session_id_size) == 0)
+		    session->internals.resumed_security_parameters.
+		    session_id_size
+		    && memcmp(session_id,
+			      session->internals.
+			      resumed_security_parameters.session_id,
+			      session_id_size) == 0)
 			return 0;
 	}
 
@@ -349,14 +292,16 @@ int _gnutls_server_restore_session(gnutls_session_t session,
 		return GNUTLS_E_INVALID_SESSION;
 	}
 
-	data = session->internals.db_retrieve_func(session->internals.db_ptr,
-						   key);
+	data =
+	    session->internals.db_retrieve_func(session->internals.db_ptr,
+						key);
 
 	if (data.data == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_SESSION;
 	}
 
+	/* expiration check is performed inside */
 	ret = gnutls_session_set_data(session, data.data, data.size);
 	gnutls_free(data.data);
 
@@ -365,7 +310,6 @@ int _gnutls_server_restore_session(gnutls_session_t session,
 		return ret;
 	}
 
-	/* expiration check is performed inside */
 	ret = _gnutls_check_resumed_params(session);
 	if (ret < 0)
 		return gnutls_assert_val(ret);
@@ -395,12 +339,12 @@ void gnutls_db_remove_session(gnutls_session_t session)
 
 	if (session->internals.db_remove_func == NULL) {
 		gnutls_assert();
-		return /* GNUTLS_E_DB_ERROR */;
+		return /* GNUTLS_E_DB_ERROR */ ;
 	}
 
 	if (session_id.data == NULL || session_id.size == 0) {
 		gnutls_assert();
-		return /* GNUTLS_E_INVALID_SESSION */;
+		return /* GNUTLS_E_INVALID_SESSION */ ;
 	}
 
 	/* if we can't read why bother writing? */
