@@ -26,24 +26,27 @@
  * and later rewritten by Nikos Mavrogiannopoulos.
  */
 
-#include <gnutls_errors.h>
-#include <gnutls_int.h>
-#include <gnutls_dtls.h>
-#include <gnutls_record.h>
+#include "errors.h"
+#include "gnutls_int.h"
+#include <dtls.h>
+#include <record.h>
 #include <ext/heartbeat.h>
-#include <gnutls_extensions.h>
+#include <extensions.h>
 #include <random.h>
 
 #ifdef ENABLE_HEARTBEAT
 /**
   * gnutls_heartbeat_enable:
-  * @session: is a #gnutls_session_t structure.
+  * @session: is a #gnutls_session_t type.
   * @type: one of the GNUTLS_HB_* flags
   *
   * If this function is called with the %GNUTLS_HB_PEER_ALLOWED_TO_SEND
-  * GnuTLS will allow heartbeat messages to be received. Moreover it also
+  * @type, GnuTLS will allow heartbeat messages to be received. Moreover it also
   * request the peer to accept heartbeat messages.
   *
+  * If the @type used is %GNUTLS_HB_LOCAL_ALLOWED_TO_SEND, then the peer
+  * will be asked to accept heartbeat messages but not send ones.
+  *
   * The function gnutls_heartbeat_allowed() can be used to test Whether
   * locally generated heartbeat messages can be accepted by the peer.
   *
@@ -53,14 +56,14 @@ void gnutls_heartbeat_enable(gnutls_session_t session, unsigned int type)
 {
 	extension_priv_data_t epriv;
 
-	epriv.num = type;
+	epriv = (void*)(intptr_t)type;
 	_gnutls_ext_set_session_data(session, GNUTLS_EXTENSION_HEARTBEAT,
 				     epriv);
 }
 
 /**
   * gnutls_heartbeat_allowed:
-  * @session: is a #gnutls_session_t structure.
+  * @session: is a #gnutls_session_t type.
   * @type: one of %GNUTLS_HB_LOCAL_ALLOWED_TO_SEND and %GNUTLS_HB_PEER_ALLOWED_TO_SEND
   *
   * This function will check whether heartbeats are allowed
@@ -74,20 +77,23 @@ int gnutls_heartbeat_allowed(gnutls_session_t session, unsigned int type)
 {
 	extension_priv_data_t epriv;
 
+	if (session->internals.handshake_in_progress != 0)
+		return 0; /* not allowed */
+
 	if (_gnutls_ext_get_session_data
 	    (session, GNUTLS_EXTENSION_HEARTBEAT, &epriv) < 0)
 		return 0;	/* Not enabled */
 
 	if (type == GNUTLS_HB_LOCAL_ALLOWED_TO_SEND) {
-		if (epriv.num & LOCAL_ALLOWED_TO_SEND)
+		if (((intptr_t)epriv) & LOCAL_ALLOWED_TO_SEND)
 			return 1;
-	} else if (epriv.num & GNUTLS_HB_PEER_ALLOWED_TO_SEND)
+	} else if (((intptr_t)epriv) & GNUTLS_HB_PEER_ALLOWED_TO_SEND)
 		return 1;
 
 	return 0;
 }
 
-#define DEFAULT_PAYLOAD_SIZE 16
+#define DEFAULT_PADDING_SIZE 16
 
 /*
  * Sends heartbeat data.
@@ -99,7 +105,7 @@ heartbeat_send_data(gnutls_session_t session, const void *data,
 	int ret, pos;
 	uint8_t *response;
 
-	response = gnutls_malloc(1 + 2 + data_size + DEFAULT_PAYLOAD_SIZE);
+	response = gnutls_malloc(1 + 2 + data_size + DEFAULT_PADDING_SIZE);
 	if (response == NULL)
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 
@@ -114,12 +120,12 @@ heartbeat_send_data(gnutls_session_t session, const void *data,
 
 	ret =
 	    gnutls_rnd(GNUTLS_RND_NONCE, &response[pos],
-		       DEFAULT_PAYLOAD_SIZE);
+		       DEFAULT_PADDING_SIZE);
 	if (ret < 0) {
 		gnutls_assert();
 		goto cleanup;
 	}
-	pos += DEFAULT_PAYLOAD_SIZE;
+	pos += DEFAULT_PADDING_SIZE;
 
 	ret =
 	    _gnutls_send_int(session, GNUTLS_HEARTBEAT, -1,
@@ -133,7 +139,7 @@ heartbeat_send_data(gnutls_session_t session, const void *data,
 
 /**
  * gnutls_heartbeat_ping:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @data_size: is the length of the ping payload.
  * @max_tries: if flags is %GNUTLS_HEARTBEAT_WAIT then this sets the number of retransmissions. Use zero for indefinite (until timeout).
  * @flags: if %GNUTLS_HEARTBEAT_WAIT then wait for pong or timeout instead of returning immediately.
@@ -174,8 +180,8 @@ gnutls_heartbeat_ping(gnutls_session_t session, size_t data_size,
 
 	switch (session->internals.hb_state) {
 	case SHB_SEND1:
-		if (data_size > DEFAULT_PAYLOAD_SIZE)
-			data_size -= DEFAULT_PAYLOAD_SIZE;
+		if (data_size > DEFAULT_PADDING_SIZE)
+			data_size -= DEFAULT_PADDING_SIZE;
 		else
 			data_size = 0;
 
@@ -188,7 +194,7 @@ gnutls_heartbeat_ping(gnutls_session_t session, size_t data_size,
 			return gnutls_assert_val(ret);
 
 		ret =
-		    _gnutls_rnd(GNUTLS_RND_NONCE,
+		    gnutls_rnd(GNUTLS_RND_NONCE,
 				session->internals.hb_local_data.data,
 				data_size);
 		if (ret < 0)
@@ -197,6 +203,8 @@ gnutls_heartbeat_ping(gnutls_session_t session, size_t data_size,
 		gettime(&session->internals.hb_ping_start);
 		session->internals.hb_local_data.length = data_size;
 		session->internals.hb_state = SHB_SEND2;
+
+		/* fallthrough */
 	case SHB_SEND2:
 		session->internals.hb_actual_retrans_timeout_ms =
 		    session->internals.hb_retrans_timeout_ms;
@@ -221,8 +229,8 @@ gnutls_heartbeat_ping(gnutls_session_t session, size_t data_size,
 
 	case SHB_RECV:
 		ret =
-		    _gnutls_recv_int(session, GNUTLS_HEARTBEAT, -1, NULL,
-				     0, NULL,
+		    _gnutls_recv_int(session, GNUTLS_HEARTBEAT,
+				     NULL, 0, NULL,
 				     session->internals.
 				     hb_actual_retrans_timeout_ms);
 		if (ret == GNUTLS_E_HEARTBEAT_PONG_RECEIVED) {
@@ -264,7 +272,7 @@ gnutls_heartbeat_ping(gnutls_session_t session, size_t data_size,
 
 /**
  * gnutls_heartbeat_pong:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @flags: should be zero
  *
  * This function replies to a ping by sending a pong to the peer.
@@ -315,7 +323,7 @@ int _gnutls_heartbeat_handle(gnutls_session_t session, mbuffer_st * bufel)
 	    (session, GNUTLS_HB_PEER_ALLOWED_TO_SEND) == 0)
 		return gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET);
 
-	if (len < 4)
+	if (len < 3 + DEFAULT_PADDING_SIZE)
 		return
 		    gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
 
@@ -323,7 +331,7 @@ int _gnutls_heartbeat_handle(gnutls_session_t session, mbuffer_st * bufel)
 	type = msg[pos++];
 
 	hb_len = _gnutls_read_uint16(&msg[pos]);
-	if (hb_len > len - 3)
+	if (hb_len > len - 3 - DEFAULT_PADDING_SIZE)
 		return
 		    gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
 
@@ -377,7 +385,7 @@ int _gnutls_heartbeat_handle(gnutls_session_t session, mbuffer_st * bufel)
 
 /**
  * gnutls_heartbeat_get_timeout:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  *
  * This function will return the milliseconds remaining
  * for a retransmission of the previously sent ping
@@ -405,7 +413,7 @@ unsigned int gnutls_heartbeat_get_timeout(gnutls_session_t session)
 
 /**
  * gnutls_heartbeat_set_timeouts:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @retrans_timeout: The time at which a retransmission will occur in milliseconds
  * @total_timeout: The time at which the connection will be aborted, in milliseconds.
  *
@@ -415,9 +423,6 @@ unsigned int gnutls_heartbeat_get_timeout(gnutls_session_t session)
  * be retransmitted. The total timeout is the time after which the
  * handshake will be aborted with %GNUTLS_E_TIMEDOUT.
  *
- * If the retransmission timeout is zero then the handshake will operate
- * in a non-blocking way, i.e., return %GNUTLS_E_AGAIN.
- *
  * Since: 3.1.2
  **/
 void gnutls_heartbeat_set_timeouts(gnutls_session_t session,
@@ -448,7 +453,7 @@ _gnutls_heartbeat_recv_params(gnutls_session_t session,
 	if (_data_size == 0)
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 
-	policy = epriv.num;
+	policy = (intptr_t)epriv;
 
 	if (data[0] == 1)
 		policy |= LOCAL_ALLOWED_TO_SEND;
@@ -458,7 +463,7 @@ _gnutls_heartbeat_recv_params(gnutls_session_t session,
 		return
 		    gnutls_assert_val(GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER);
 
-	epriv.num = policy;
+	epriv = (void*)(intptr_t)policy;
 	_gnutls_ext_set_session_data(session, GNUTLS_EXTENSION_HEARTBEAT,
 				     epriv);
 
@@ -476,7 +481,7 @@ _gnutls_heartbeat_send_params(gnutls_session_t session,
 	    (session, GNUTLS_EXTENSION_HEARTBEAT, &epriv) < 0)
 		return 0;	/* nothing to send - not enabled */
 
-	if (epriv.num & GNUTLS_HB_PEER_ALLOWED_TO_SEND)
+	if (((intptr_t)epriv) & GNUTLS_HB_PEER_ALLOWED_TO_SEND)
 		p = 1;
 	else			/*if (epriv.num & GNUTLS_HB_PEER_NOT_ALLOWED_TO_SEND) */
 		p = 2;
@@ -492,7 +497,7 @@ _gnutls_heartbeat_pack(extension_priv_data_t epriv, gnutls_buffer_st * ps)
 {
 	int ret;
 
-	BUFFER_APPEND_NUM(ps, epriv.num);
+	BUFFER_APPEND_NUM(ps, (intptr_t)epriv);
 
 	return 0;
 
@@ -505,7 +510,7 @@ _gnutls_heartbeat_unpack(gnutls_buffer_st * ps,
 	extension_priv_data_t epriv;
 	int ret;
 
-	BUFFER_POP_NUM(ps, epriv.num);
+	BUFFER_POP_CAST_NUM(ps, epriv);
 
 	*_priv = epriv;
 
@@ -514,8 +519,8 @@ _gnutls_heartbeat_unpack(gnutls_buffer_st * ps,
 	return ret;
 }
 
-extension_entry_st ext_mod_heartbeat = {
-	.name = "HEARTBEAT",
+const extension_entry_st ext_mod_heartbeat = {
+	.name = "Heartbeat",
 	.type = GNUTLS_EXTENSION_HEARTBEAT,
 	.parse_type = GNUTLS_EXT_TLS,
 
