@@ -21,9 +21,10 @@
  */
 
 #include "gnutls_int.h"
-#include "gnutls_auth.h"
-#include "gnutls_errors.h"
-#include "gnutls_num.h"
+#include "auth.h"
+#include "errors.h"
+#include "num.h"
+#include "str.h"
 #include <ext/server_name.h>
 
 static int _gnutls_server_name_recv_params(gnutls_session_t session,
@@ -38,9 +39,13 @@ static int _gnutls_server_name_pack(extension_priv_data_t _priv,
 				    gnutls_buffer_st * ps);
 static void _gnutls_server_name_deinit_data(extension_priv_data_t priv);
 
+int
+_gnutls_server_name_set_raw(gnutls_session_t session,
+		       gnutls_server_name_type_t type,
+		       const void *name, size_t name_length);
 
-extension_entry_st ext_mod_server_name = {
-	.name = "SERVER NAME",
+const extension_entry_st ext_mod_server_name = {
+	.name = "Server Name Indication",
 	.type = GNUTLS_EXTENSION_SERVER_NAME,
 	.parse_type = GNUTLS_EXT_APPLICATION,
 
@@ -64,7 +69,7 @@ static int
 _gnutls_server_name_recv_params(gnutls_session_t session,
 				const uint8_t * data, size_t _data_size)
 {
-	int i;
+	int i, j;
 	const unsigned char *p;
 	uint16_t len, type;
 	ssize_t data_size = _data_size;
@@ -124,10 +129,8 @@ _gnutls_server_name_recv_params(gnutls_session_t session,
 			return GNUTLS_E_MEMORY_ERROR;
 		}
 
-		priv->server_names_size = server_names;
-
 		p = data + 2;
-		for (i = 0; i < server_names; i++) {
+		for (j = i = 0; i < server_names; i++) {
 			type = *p;
 			p++;
 
@@ -136,13 +139,18 @@ _gnutls_server_name_recv_params(gnutls_session_t session,
 
 			switch (type) {
 			case 0:	/* NAME_DNS */
-				if (len <= MAX_SERVER_NAME_SIZE) {
-					memcpy(priv->server_names[i].name,
+				if (len < MAX_SERVER_NAME_SIZE) {
+					memcpy(priv->server_names[j].name,
 					       p, len);
-					priv->server_names[i].name_length =
-					    len;
-					priv->server_names[i].type =
-					    GNUTLS_NAME_DNS;
+					priv->server_names[j].name[len] = 0;
+					priv->server_names[j].name_length =
+					    strlen((char*)priv->server_names[j].name);
+					if (priv->server_names[j].name_length == len) {
+						/* valid ascii with no embedded NULL */
+						priv->server_names[j].type =
+						    GNUTLS_NAME_DNS;
+						j++;
+					}
 					break;
 				}
 			}
@@ -151,7 +159,9 @@ _gnutls_server_name_recv_params(gnutls_session_t session,
 			p += len;
 		}
 
-		epriv.ptr = priv;
+		priv->server_names_size = j;
+
+		epriv = priv;
 		_gnutls_ext_set_session_data(session,
 					     GNUTLS_EXTENSION_SERVER_NAME,
 					     epriv);
@@ -180,11 +190,10 @@ _gnutls_server_name_send_params(gnutls_session_t session,
 	if (ret < 0)
 		return 0;
 
-
 	/* this function sends the client extension data (dnsname)
 	 */
 	if (session->security_parameters.entity == GNUTLS_CLIENT) {
-		priv = epriv.ptr;
+		priv = epriv;
 
 		if (priv->server_names_size == 0)
 			return 0;
@@ -228,6 +237,8 @@ _gnutls_server_name_send_params(gnutls_session_t session,
 				if (ret < 0)
 					return gnutls_assert_val(ret);
 
+				_gnutls_debug_log("HSK[%p]: sent server name: '%s'\n", session, priv->server_names[i].name);
+
 				ret =
 				    _gnutls_buffer_append_data_prefix
 				    (extdata, 16,
@@ -248,7 +259,7 @@ _gnutls_server_name_send_params(gnutls_session_t session,
 
 /**
  * gnutls_server_name_get:
- * @session: is a #gnutls_session_t structure.
+ * @session: is a #gnutls_session_t type.
  * @data: will hold the data
  * @data_length: will hold the data length. Must hold the maximum size of data.
  * @type: will hold the server name indicator type
@@ -260,7 +271,7 @@ _gnutls_server_name_send_params(gnutls_session_t session,
  *
  * If @type is GNUTLS_NAME_DNS, then this function is to be used by
  * servers that support virtual hosting, and the data will be a null
- * terminated UTF-8 string.
+ * terminated IDNA ACE string (prior to GnuTLS 3.4.0 it was a UTF-8 string).
  *
  * If @data has not enough size to hold the server name
  * GNUTLS_E_SHORT_MEMORY_BUFFER is returned, and @data_length will
@@ -271,8 +282,9 @@ _gnutls_server_name_send_params(gnutls_session_t session,
  * and so on.  If no name with the given index exists
  * GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE is returned.
  *
- * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned,
- *   otherwise a negative error code is returned.
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, on UTF-8
+ *  decoding error %GNUTLS_E_IDNA_ERROR is returned, otherwise a negative
+ *  error code is returned.
  **/
 int
 gnutls_server_name_get(gnutls_session_t session, void *data,
@@ -282,6 +294,7 @@ gnutls_server_name_get(gnutls_session_t session, void *data,
 	char *_data = data;
 	server_name_ext_st *priv;
 	int ret;
+	gnutls_datum_t idn_name = {NULL,0};
 	extension_priv_data_t epriv;
 
 	if (session->security_parameters.entity == GNUTLS_CLIENT) {
@@ -298,7 +311,7 @@ gnutls_server_name_get(gnutls_session_t session, void *data,
 		return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
 	}
 
-	priv = epriv.ptr;
+	priv = epriv;
 
 	if (indx + 1 > priv->server_names_size) {
 		return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
@@ -306,44 +319,35 @@ gnutls_server_name_get(gnutls_session_t session, void *data,
 
 	*type = priv->server_names[indx].type;
 
+	ret = gnutls_idna_map((char*)priv->server_names[indx].name, priv->server_names[indx].name_length, &idn_name, 0);
+	if (ret < 0) {
+		 _gnutls_debug_log("unable to convert name %s to IDNA2003 format\n", (char*)priv->server_names[indx].name);
+		 return GNUTLS_E_IDNA_ERROR;
+	}
+
 	if (*data_length >	/* greater since we need one extra byte for the null */
-	    priv->server_names[indx].name_length) {
-		*data_length = priv->server_names[indx].name_length;
-		memcpy(data, priv->server_names[indx].name, *data_length);
+	    idn_name.size) {
+		*data_length = idn_name.size;
+		memcpy(data, idn_name.data, *data_length);
 
 		if (*type == GNUTLS_NAME_DNS)	/* null terminate */
 			_data[(*data_length)] = 0;
 
 	} else {
-		*data_length = priv->server_names[indx].name_length + 1;
-		return GNUTLS_E_SHORT_MEMORY_BUFFER;
+		*data_length = idn_name.size + 1;
+		ret = GNUTLS_E_SHORT_MEMORY_BUFFER;
+		goto cleanup;
 	}
 
-	return 0;
+	ret = 0;
+ cleanup:
+	gnutls_free(idn_name.data);
+	return ret;
 }
 
-/**
- * gnutls_server_name_set:
- * @session: is a #gnutls_session_t structure.
- * @type: specifies the indicator type
- * @name: is a string that contains the server name.
- * @name_length: holds the length of name
- *
- * This function is to be used by clients that want to inform (via a
- * TLS extension mechanism) the server of the name they connected to.
- * This should be used by clients that connect to servers that do
- * virtual hosting.
- *
- * The value of @name depends on the @type type.  In case of
- * %GNUTLS_NAME_DNS, an ASCII (0)-terminated domain name string,
- * without the trailing dot, is expected.  IPv4 or IPv6 addresses are
- * not permitted.
- *
- * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned,
- *   otherwise a negative error code is returned.
- **/
+/* This does not do any conversion not perform any check */
 int
-gnutls_server_name_set(gnutls_session_t session,
+_gnutls_server_name_set_raw(gnutls_session_t session,
 		       gnutls_server_name_type_t type,
 		       const void *name, size_t name_length)
 {
@@ -352,13 +356,9 @@ gnutls_server_name_set(gnutls_session_t session,
 	extension_priv_data_t epriv;
 	int set = 0;
 
-	if (session->security_parameters.entity == GNUTLS_SERVER) {
-		gnutls_assert();
-		return GNUTLS_E_INVALID_REQUEST;
-	}
-
-	if (name_length > MAX_SERVER_NAME_SIZE)
+	if (name_length > MAX_SERVER_NAME_SIZE) {
 		return GNUTLS_E_SHORT_MEMORY_BUFFER;
+	}
 
 	ret =
 	    _gnutls_ext_get_session_data(session,
@@ -374,9 +374,9 @@ gnutls_server_name_set(gnutls_session_t session,
 			gnutls_assert();
 			return GNUTLS_E_MEMORY_ERROR;
 		}
-		epriv.ptr = priv;
+		epriv = priv;
 	} else
-		priv = epriv.ptr;
+		priv = epriv;
 
 	server_names = priv->server_names_size + 1;
 
@@ -384,8 +384,12 @@ gnutls_server_name_set(gnutls_session_t session,
 		server_names = MAX_SERVER_NAME_EXTENSIONS;
 
 	priv->server_names[server_names - 1].type = type;
-	memcpy(priv->server_names[server_names - 1].name, name,
-	       name_length);
+
+	if (name_length > 0) {
+		memcpy(priv->server_names[server_names - 1].name, name,
+		       name_length);
+		priv->server_names[server_names - 1].name[name_length] = 0;
+	}
 	priv->server_names[server_names - 1].name_length = name_length;
 
 	priv->server_names_size = server_names;
@@ -398,16 +402,72 @@ gnutls_server_name_set(gnutls_session_t session,
 	return 0;
 }
 
+/**
+ * gnutls_server_name_set:
+ * @session: is a #gnutls_session_t type.
+ * @type: specifies the indicator type
+ * @name: is a string that contains the server name.
+ * @name_length: holds the length of name
+ *
+ * This function is to be used by clients that want to inform (via a
+ * TLS extension mechanism) the server of the name they connected to.
+ * This should be used by clients that connect to servers that do
+ * virtual hosting.
+ *
+ * The value of @name depends on the @type type.  In case of
+ * %GNUTLS_NAME_DNS, a UTF-8 null-terminated domain name string,
+ * without the trailing dot, is expected.
+ *
+ * IPv4 or IPv6 addresses are not permitted to be set by this function.
+ * If the function is called with a name of @name_length zero it will clear
+ * all server names set.
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned,
+ *   otherwise a negative error code is returned.
+ **/
+int
+gnutls_server_name_set(gnutls_session_t session,
+		       gnutls_server_name_type_t type,
+		       const void *name, size_t name_length)
+{
+	int ret;
+	gnutls_datum_t idn_name = {NULL,0};
+
+	if (session->security_parameters.entity == GNUTLS_SERVER) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	if (name_length == 0) { /* unset extension */
+		_gnutls_ext_unset_session_data(session, GNUTLS_EXTENSION_SERVER_NAME);
+		return 0;
+	}
+
+	ret = gnutls_idna_map(name, name_length, &idn_name, 0);
+	if (ret < 0) {
+		 _gnutls_debug_log("unable to convert name %s to IDNA2003 format\n", (char*)name);
+		 return ret;
+	}
+
+	name = idn_name.data;
+	name_length = idn_name.size;
+
+	ret = _gnutls_server_name_set_raw(session, type, name, name_length);
+	gnutls_free(idn_name.data);
+
+	return ret;
+}
+
 static void _gnutls_server_name_deinit_data(extension_priv_data_t priv)
 {
-	gnutls_free(priv.ptr);
+	gnutls_free(priv);
 }
 
 static int
 _gnutls_server_name_pack(extension_priv_data_t epriv,
 			 gnutls_buffer_st * ps)
 {
-	server_name_ext_st *priv = epriv.ptr;
+	server_name_ext_st *priv = epriv;
 	unsigned int i;
 	int ret;
 
@@ -448,7 +508,7 @@ _gnutls_server_name_unpack(gnutls_buffer_st * ps,
 			   priv->server_names[i].name_length);
 	}
 
-	epriv.ptr = priv;
+	epriv = priv;
 	*_priv = epriv;
 
 	return 0;
