@@ -16,38 +16,39 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
 
-#ifndef GNUTLS_LIB_HASH_INT_H
-#define GNUTLS_LIB_HASH_INT_H
+#ifndef GNUTLS_HASH_INT_H
+#define GNUTLS_HASH_INT_H
 
 #include "gnutls_int.h"
 #include <gnutls/crypto.h>
-#include "crypto-backend.h"
-#include "crypto.h"
+#include <crypto-backend.h>
+#include <crypto.h>
 
 /* for message digests */
 
 extern int crypto_mac_prio;
-extern int crypto_digest_prio;
+extern gnutls_crypto_mac_st _gnutls_mac_ops;
 
-typedef int (*hash_func)(void *handle, const void *text, size_t size);
-typedef int (*nonce_func)(void *handle, const void *text, size_t size);
-typedef int (*output_func)(void *src_ctx, void *digest, size_t digestsize);
-typedef void (*hash_deinit_func)(void *handle);
-typedef void *(*copy_func)(const void *handle);
-typedef int (*setkey_func)(void *handle, const void *key, size_t keysize);
+extern int crypto_digest_prio;
+extern gnutls_crypto_digest_st _gnutls_digest_ops;
+
+typedef int (*hash_func) (void *handle, const void *text, size_t size);
+typedef int (*nonce_func) (void *handle, const void *text, size_t size);
+typedef int (*output_func) (void *src_ctx, void *digest,
+			    size_t digestsize);
+typedef void (*hash_deinit_func) (void *handle);
 
 typedef struct {
 	const mac_entry_st *e;
 	hash_func hash;
 	output_func output;
 	hash_deinit_func deinit;
-	copy_func copy;
 
-	const void *key; /* esoteric use by SSL3 MAC functions */
+	const void *key;	/* esoteric use by SSL3 MAC functions */
 	int keysize;
 
 	void *handle;
@@ -61,8 +62,6 @@ typedef struct {
 	nonce_func setnonce;
 	output_func output;
 	hash_deinit_func deinit;
-	copy_func copy;
-	setkey_func setkey;
 
 	void *handle;
 } mac_hd_st;
@@ -71,17 +70,15 @@ typedef struct {
 int _gnutls_digest_exists(gnutls_digest_algorithm_t algo);
 
 int _gnutls_mac_exists(gnutls_mac_algorithm_t algorithm);
-int _gnutls_mac_init(mac_hd_st *, const mac_entry_st *e, const void *key,
-		     int keylen);
-
-int _gnutls_mac_copy(const mac_hd_st *handle, mac_hd_st *dst);
+int _gnutls_mac_init(mac_hd_st *, const mac_entry_st * e,
+		     const void *key, int keylen);
 
 int _gnutls_mac_fast(gnutls_mac_algorithm_t algorithm, const void *key,
 		     int keylen, const void *text, size_t textlen,
 		     void *digest);
 
-inline static int _gnutls_mac(mac_hd_st *handle, const void *text,
-			      size_t textlen)
+inline static int
+_gnutls_mac(mac_hd_st * handle, const void *text, size_t textlen)
 {
 	if (textlen > 0) {
 		return handle->hash(handle->handle, text, textlen);
@@ -89,34 +86,28 @@ inline static int _gnutls_mac(mac_hd_st *handle, const void *text,
 	return 0;
 }
 
-inline static void _gnutls_mac_output(mac_hd_st *handle, void *digest)
+inline static void _gnutls_mac_output(mac_hd_st * handle, void *digest)
 {
 	if (digest != NULL) {
 		handle->output(handle->handle, digest, handle->mac_len);
 	}
 }
 
-inline static int _gnutls_mac_set_nonce(mac_hd_st *handle, const void *nonce,
-					size_t n_size)
+inline static int
+_gnutls_mac_set_nonce(mac_hd_st * handle, const void *nonce, size_t n_size)
 {
 	if (handle->setnonce)
 		return handle->setnonce(handle->handle, nonce, n_size);
 	return 0;
 }
 
-inline static int _gnutls_mac_setkey(mac_hd_st *handle, const void *key,
-				     size_t key_size)
-{
-	return handle->setkey(handle->handle, key, key_size);
-}
-
-void _gnutls_mac_deinit(mac_hd_st *handle, void *digest);
+void _gnutls_mac_deinit(mac_hd_st * handle, void *digest);
 
 /* Hash interface */
-int _gnutls_hash_init(digest_hd_st *, const mac_entry_st *e);
+int _gnutls_hash_init(digest_hd_st *, const mac_entry_st * e);
 
-inline static int _gnutls_hash(digest_hd_st *handle, const void *text,
-			       size_t textlen)
+inline static int
+_gnutls_hash(digest_hd_st * handle, const void *text, size_t textlen)
 {
 	if (textlen > 0) {
 		return handle->hash(handle->handle, text, textlen);
@@ -127,29 +118,30 @@ inline static int _gnutls_hash(digest_hd_st *handle, const void *text,
 /* when the current output is needed without calling deinit
  */
 #define _gnutls_hash_output(h, d) \
-	(h)->output((h)->handle, d, _gnutls_hash_get_algo_len((h)->e))
+  (h)->output((h)->handle, d, _gnutls_hash_get_algo_len((h)->e))
 
-void _gnutls_hash_deinit(digest_hd_st *handle, void *digest);
+void _gnutls_hash_deinit(digest_hd_st * handle, void *digest);
 
-int _gnutls_hash_copy(const digest_hd_st *handle, digest_hd_st *dst);
-
-int _gnutls_hash_fast(gnutls_digest_algorithm_t algorithm, const void *text,
-		      size_t textlen, void *digest);
-
-int _gnutls_hash_squeeze(digest_hd_st *handle, void *output, size_t length);
+int
+_gnutls_hash_fast(gnutls_digest_algorithm_t algorithm,
+		  const void *text, size_t textlen, void *digest);
 
 #ifdef ENABLE_SSL3
 /* helper functions */
-int _gnutls_mac_init_ssl3(digest_hd_st *, const mac_entry_st *e, void *key,
-			  int keylen);
-int _gnutls_mac_deinit_ssl3(digest_hd_st *handle, void *digest);
-int _gnutls_mac_output_ssl3(digest_hd_st *handle, void *digest);
+int _gnutls_mac_init_ssl3(digest_hd_st *, const mac_entry_st * e,
+			  void *key, int keylen);
+int _gnutls_mac_deinit_ssl3(digest_hd_st * handle, void *digest);
+int _gnutls_mac_output_ssl3(digest_hd_st * handle, void *digest);
 
-int _gnutls_ssl3_generate_random(void *secret, int secret_len, void *rnd,
-				 int random_len, int bytes, uint8_t *ret);
+int _gnutls_ssl3_generate_random(void *secret, int secret_len,
+				 void *rnd, int random_len, int bytes,
+				 uint8_t * ret);
+int _gnutls_ssl3_hash_md5(const void *first, int first_len,
+			  const void *second, int second_len,
+			  int ret_len, uint8_t * ret);
 
-int _gnutls_mac_deinit_ssl3_handshake(digest_hd_st *handle, void *digest,
-				      uint8_t *key, uint32_t key_size);
+int _gnutls_mac_deinit_ssl3_handshake(digest_hd_st * handle, void *digest,
+				      uint8_t * key, uint32_t key_size);
 #endif
 
 inline static int IS_SHA(gnutls_digest_algorithm_t algo)
@@ -161,4 +153,4 @@ inline static int IS_SHA(gnutls_digest_algorithm_t algo)
 	return 0;
 }
 
-#endif /* GNUTLS_LIB_HASH_INT_H */
+#endif				/* GNUTLS_HASH_INT_H */
