@@ -16,36 +16,62 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
 
-#ifndef GNUTLS_LIB_NUM_H
-#define GNUTLS_LIB_NUM_H
+#ifndef GNUTLS_NUM_H
+#define GNUTLS_NUM_H
 
 #include "gnutls_int.h"
 
 #include <minmax.h>
 #include <byteswap.h>
 
-/* data should be at least 3 bytes */
-inline static uint32_t _gnutls_read_uint24(const uint8_t *data)
+int _gnutls_uint64pp(uint64 *);
+int _gnutls_uint48pp(uint64 *);
+
+#define UINT64DATA(x) ((x).i)
+
+inline static uint32_t _gnutls_uint24touint32(uint24 num)
 {
-	return (data[0] << 16) | (data[1] << 8) | (data[2]);
+	uint32_t ret = 0;
+
+	((uint8_t *) & ret)[1] = num.pint[0];
+	((uint8_t *) & ret)[2] = num.pint[1];
+	((uint8_t *) & ret)[3] = num.pint[2];
+	return ret;
 }
 
-inline static uint64_t _gnutls_read_uint64(const uint8_t *data)
+inline static uint24 _gnutls_uint32touint24(uint32_t num)
 {
-	uint64_t res;
+	uint24 ret;
 
-	memcpy(&res, data, sizeof(uint64_t));
+	ret.pint[0] = ((uint8_t *) & num)[1];
+	ret.pint[1] = ((uint8_t *) & num)[2];
+	ret.pint[2] = ((uint8_t *) & num)[3];
+	return ret;
+
+}
+
+/* data should be at least 3 bytes */
+inline static uint32_t _gnutls_read_uint24(const uint8_t * data)
+{
+	uint32_t res;
+	uint24 num;
+
+	num.pint[0] = data[0];
+	num.pint[1] = data[1];
+	num.pint[2] = data[2];
+
+	res = _gnutls_uint24touint32(num);
 #ifndef WORDS_BIGENDIAN
-	res = bswap_64(res);
+	res = bswap_32(res);
 #endif
 	return res;
 }
 
-inline static void _gnutls_write_uint64(uint64_t num, uint8_t *data)
+inline static void _gnutls_write_uint64(uint64_t num, uint8_t * data)
 {
 #ifndef WORDS_BIGENDIAN
 	num = bswap_64(num);
@@ -53,14 +79,21 @@ inline static void _gnutls_write_uint64(uint64_t num, uint8_t *data)
 	memcpy(data, &num, 8);
 }
 
-inline static void _gnutls_write_uint24(uint32_t num, uint8_t *data)
+inline static void _gnutls_write_uint24(uint32_t num, uint8_t * data)
 {
-	data[0] = num >> 16;
-	data[1] = num >> 8;
-	data[2] = num;
+	uint24 tmp;
+
+#ifndef WORDS_BIGENDIAN
+	num = bswap_32(num);
+#endif
+	tmp = _gnutls_uint32touint24(num);
+
+	data[0] = tmp.pint[0];
+	data[1] = tmp.pint[1];
+	data[2] = tmp.pint[2];
 }
 
-inline static uint32_t _gnutls_read_uint32(const uint8_t *data)
+inline static uint32_t _gnutls_read_uint32(const uint8_t * data)
 {
 	uint32_t res;
 
@@ -71,15 +104,16 @@ inline static uint32_t _gnutls_read_uint32(const uint8_t *data)
 	return res;
 }
 
-inline static void _gnutls_write_uint32(uint32_t num, uint8_t *data)
+inline static void _gnutls_write_uint32(uint32_t num, uint8_t * data)
 {
+
 #ifndef WORDS_BIGENDIAN
 	num = bswap_32(num);
 #endif
 	memcpy(data, &num, sizeof(uint32_t));
 }
 
-inline static uint16_t _gnutls_read_uint16(const uint8_t *data)
+inline static uint16_t _gnutls_read_uint16(const uint8_t * data)
 {
 	uint16_t res;
 	memcpy(&res, data, sizeof(uint16_t));
@@ -89,8 +123,9 @@ inline static uint16_t _gnutls_read_uint16(const uint8_t *data)
 	return res;
 }
 
-inline static void _gnutls_write_uint16(uint16_t num, uint8_t *data)
+inline static void _gnutls_write_uint16(uint16_t num, uint8_t * data)
 {
+
 #ifndef WORDS_BIGENDIAN
 	num = bswap_16(num);
 #endif
@@ -115,4 +150,16 @@ inline static uint16_t _gnutls_conv_uint16(uint16_t data)
 #endif
 }
 
-#endif /* GNUTLS_LIB_NUM_H */
+inline static uint32_t _gnutls_uint64touint32(const uint64 * num)
+{
+	uint32_t ret;
+
+	memcpy(&ret, &num->i[4], 4);
+#ifndef WORDS_BIGENDIAN
+	ret = bswap_32(ret);
+#endif
+
+	return ret;
+}
+
+#endif				/* GNUTLS_NUM_H */
