@@ -23,20 +23,20 @@
 /* Functions on keyring parsing
  */
 
-#include <gnutls_int.h>
-#include <gnutls_datum.h>
-#include <gnutls_global.h>
-#include <gnutls_errors.h>
+#include "gnutls_int.h"
+#include <datum.h>
+#include <global.h>
+#include "errors.h"
 #include <openpgp_int.h>
-#include <gnutls_openpgp.h>
-#include <gnutls_num.h>
+#include <openpgp.h>
+#include <num.h>
 
 /* Keyring stuff.
  */
 
 /**
  * gnutls_openpgp_keyring_init:
- * @keyring: The structure to be initialized
+ * @keyring: A pointer to the type to be initialized
  *
  * This function will initialize an keyring structure.
  *
@@ -54,7 +54,7 @@ int gnutls_openpgp_keyring_init(gnutls_openpgp_keyring_t * keyring)
 
 /**
  * gnutls_openpgp_keyring_deinit:
- * @keyring: The structure to be initialized
+ * @keyring: A pointer to the type to be initialized
  *
  * This function will deinitialize a keyring structure.
  **/
@@ -124,6 +124,7 @@ gnutls_openpgp_keyring_import(gnutls_openpgp_keyring_t keyring,
 	cdk_stream_t input = NULL;
 	size_t raw_len = 0;
 	uint8_t *raw_data = NULL;
+	unsigned free_data = 0;
 
 	if (data->data == NULL || data->size == 0) {
 		gnutls_assert();
@@ -139,7 +140,7 @@ gnutls_openpgp_keyring_import(gnutls_openpgp_keyring_t keyring,
 	 * not thread safe.
 	 */
 	if (format == GNUTLS_OPENPGP_FMT_BASE64) {
-		size_t written = 0;
+		size_t seen = 0;
 
 		err =
 		    cdk_stream_tmp_from_mem(data->data, data->size,
@@ -168,15 +169,22 @@ gnutls_openpgp_keyring_import(gnutls_openpgp_keyring_t keyring,
 
 		do {
 			err =
-			    cdk_stream_read(input, raw_data + written,
-					    raw_len - written);
+			    cdk_stream_read(input, raw_data + seen,
+					    raw_len - seen);
 
 			if (err > 0)
-				written += err;
+				seen += err;
 		}
-		while (written < raw_len && err != EOF && err > 0);
+		while (seen < raw_len && err != EOF && err > 0);
 
-		raw_len = written;
+		raw_len = seen;
+		if (raw_len == 0) {
+			gnutls_assert();
+			err = GNUTLS_E_BASE64_DECODING_ERROR;
+			goto error;
+		}
+
+		free_data = 1;
 	} else {		/* RAW */
 		raw_len = data->size;
 		raw_data = data->data;
@@ -187,6 +195,11 @@ gnutls_openpgp_keyring_import(gnutls_openpgp_keyring_t keyring,
 	if (err)
 		gnutls_assert();
 
+	if (free_data) {
+		err = _gnutls_map_cdk_rc(err);
+		goto error;
+	}
+
 	return _gnutls_map_cdk_rc(err);
 
       error:
@@ -246,7 +259,7 @@ int gnutls_openpgp_keyring_get_crt_count(gnutls_openpgp_keyring_t ring)
  * gnutls_openpgp_keyring_get_crt:
  * @ring: Holds the keyring.
  * @idx: the index of the certificate to export
- * @cert: An uninitialized #gnutls_openpgp_crt_t structure
+ * @cert: An uninitialized #gnutls_openpgp_crt_t type
  *
  * This function will extract an OpenPGP certificate from the given
  * keyring.  If the index given is out of range
