@@ -23,18 +23,18 @@
 /* Functions on OpenPGP key parsing
  */
 
-#include <gnutls_int.h>
-#include <gnutls_datum.h>
-#include <gnutls_global.h>
-#include <gnutls_errors.h>
+#include "gnutls_int.h"
+#include <datum.h>
+#include <global.h>
+#include "errors.h"
 #include <openpgp_int.h>
-#include <gnutls_str.h>
-#include <gnutls_num.h>
+#include <str.h>
+#include <num.h>
 #include <x509/common.h>
 
 /**
  * gnutls_openpgp_crt_init:
- * @key: The structure to be initialized
+ * @key: A pointer to the type to be initialized
  *
  * This function will initialize an OpenPGP key structure.
  *
@@ -51,7 +51,7 @@ int gnutls_openpgp_crt_init(gnutls_openpgp_crt_t * key)
 
 /**
  * gnutls_openpgp_crt_deinit:
- * @key: The structure to be initialized
+ * @key: A pointer to the type to be initialized
  *
  * This function will deinitialize a key structure.
  **/
@@ -169,7 +169,16 @@ _gnutls_openpgp_export(cdk_kbnode_t node,
 
 	if (format == GNUTLS_OPENPGP_FMT_BASE64) {
 		unsigned char *in = gnutls_calloc(1, *output_data_size);
-		memcpy(in, output_data, *output_data_size);
+		if (in == NULL)
+			return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+
+		rc = cdk_kbnode_write_to_mem(node, in, output_data_size);
+		if (rc) {
+			gnutls_free(in);
+			rc = _gnutls_map_cdk_rc(rc);
+			gnutls_assert();
+			return rc;
+		}
 
 		/* Calculate the size of the encoded data and check if the provided
 		   buffer is large enough. */
@@ -574,7 +583,7 @@ int gnutls_openpgp_crt_get_revoked_status(gnutls_openpgp_crt_t key)
 
 /**
  * gnutls_openpgp_crt_check_hostname:
- * @key: should contain a #gnutls_openpgp_crt_t structure
+ * @key: should contain a #gnutls_openpgp_crt_t type
  * @hostname: A null terminated string that contains a DNS name
  *
  * This function will check if the given key's owner matches the
@@ -586,6 +595,28 @@ int gnutls_openpgp_crt_get_revoked_status(gnutls_openpgp_crt_t key)
 int
 gnutls_openpgp_crt_check_hostname(gnutls_openpgp_crt_t key,
 				  const char *hostname)
+{
+	return gnutls_openpgp_crt_check_hostname2(key, hostname, 0);
+}
+
+/**
+ * gnutls_openpgp_crt_check_hostname2:
+ * @key: should contain a #gnutls_openpgp_crt_t type
+ * @hostname: A null terminated string that contains a DNS name
+ * @flags: gnutls_certificate_verify_flags
+ *
+ * This function will check if the given key's owner matches the
+ * given hostname. 
+ *
+ * Unless, the flag %GNUTLS_VERIFY_DO_NOT_ALLOW_WILDCARDS is specified,
+ * wildcards are only considered if the domain name consists of three
+ * components or more, and the wildcard starts at the leftmost position.
+ *
+ * Returns: non-zero for a successful match, and zero on failure.
+ **/
+int
+gnutls_openpgp_crt_check_hostname2(gnutls_openpgp_crt_t key,
+				  const char *hostname, unsigned flags)
 {
 	char dnsname[MAX_CN];
 	size_t dnsnamesize;
@@ -605,7 +636,49 @@ gnutls_openpgp_crt_check_hostname(gnutls_openpgp_crt_t key,
 			dnsnamesize--;
 
 			if (_gnutls_hostname_compare
-			    (dnsname, dnsnamesize, hostname, 0))
+			    (dnsname, dnsnamesize, hostname, flags))
+				return 1;
+		}
+	}
+
+	/* not found a matching name */
+	return 0;
+}
+
+/**
+ * gnutls_openpgp_crt_check_email:
+ * @key: should contain a #gnutls_openpgp_crt_t type
+ * @email: A null terminated string that contains an RFC822 address (email)
+ * @flags: gnutls_certificate_verify_flags
+ *
+ * This function will check if the given key's owner matches the
+ * given email address. 
+ *
+ * Returns: non-zero for a successful match, and zero on failure.
+ **/
+int
+gnutls_openpgp_crt_check_email(gnutls_openpgp_crt_t key,
+				  const char *email, unsigned flags)
+{
+	char rfc822name[MAX_CN];
+	size_t rfc822name_size;
+	int ret = 0;
+	int i;
+
+	/* Check through all included names. */
+	for (i = 0; !(ret < 0); i++) {
+		rfc822name_size = sizeof(rfc822name);
+		ret =
+		    gnutls_openpgp_crt_get_name(key, i, rfc822name,
+						&rfc822name_size);
+
+		if (ret == 0) {
+			/* Length returned by gnutls_openpgp_crt_get_name includes
+			   the terminating (0). */
+			rfc822name_size--;
+
+			if (_gnutls_hostname_compare
+			    (rfc822name, rfc822name_size, email, GNUTLS_VERIFY_DO_NOT_ALLOW_WILDCARDS))
 				return 1;
 		}
 	}
@@ -634,7 +707,7 @@ unsigned int _gnutls_get_pgp_key_usage(unsigned int cdk_usage)
 
 /**
  * gnutls_openpgp_crt_get_key_usage:
- * @key: should contain a gnutls_openpgp_crt_t structure
+ * @key: should contain a gnutls_openpgp_crt_t type
  * @key_usage: where the key usage bits will be stored
  *
  * This function will return certificate's key usage, by checking the
@@ -1085,7 +1158,7 @@ gnutls_openpgp_crt_get_subkey_idx(gnutls_openpgp_crt_t key,
 
 /**
  * gnutls_openpgp_crt_get_subkey_usage:
- * @key: should contain a gnutls_openpgp_crt_t structure
+ * @key: should contain a gnutls_openpgp_crt_t type
  * @idx: the subkey index
  * @key_usage: where the key usage bits will be stored
  *
@@ -1194,7 +1267,7 @@ _gnutls_read_pgp_mpi(cdk_packet_t pkt, unsigned int priv, size_t idx,
 		return _gnutls_map_cdk_rc(err);
 	}
 
-	err = _gnutls_mpi_scan(m, buf, buf_size);
+	err = _gnutls_mpi_init_scan(m, buf, buf_size);
 	gnutls_free(buf);
 
 	if (err < 0) {
@@ -1560,16 +1633,16 @@ int
 gnutls_openpgp_crt_get_preferred_key_id(gnutls_openpgp_crt_t key,
 					gnutls_openpgp_keyid_t keyid)
 {
-	if (!key->preferred_set)
-		return
-		    gnutls_assert_val
-		    (GNUTLS_E_OPENPGP_PREFERRED_KEY_ERROR);
-
 	if (!key || !keyid) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	if (!key->preferred_set)
+		return
+		    gnutls_assert_val
+		    (GNUTLS_E_OPENPGP_PREFERRED_KEY_ERROR);
+
 	memcpy(keyid, key->preferred_keyid, GNUTLS_OPENPGP_KEYID_SIZE);
 
 	return 0;
