@@ -16,16 +16,11 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
  *
  */
 
-#ifndef GNUTLS_LIB_SESSION_PACK_H
-# define GNUTLS_LIB_SESSION_PACK_H
-
 int _gnutls_session_pack(gnutls_session_t session,
 			 gnutls_datum_t * packed_session);
 int _gnutls_session_unpack(gnutls_session_t session,
 			   const gnutls_datum_t * packed_session);
-
-#endif				/* GNUTLS_LIB_SESSION_PACK_H */
