@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2003-2012 Free Software Foundation, Inc.
+ * Copyright (C) 2003-2016 Free Software Foundation, Inc.
+ * Copyright (C) 2015-2016 Red Hat, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -20,20 +21,41 @@
  *
  */
 
-#include <gnutls_int.h>
+#include "gnutls_int.h"
 #include <libtasn1.h>
 
-#include <gnutls_datum.h>
-#include <gnutls_global.h>
-#include <gnutls_errors.h>
+#include <datum.h>
+#include <global.h>
+#include "errors.h"
 #include <common.h>
 #include <x509_b64.h>
 #include <x509_int.h>
-#include <gnutls_x509.h>
+#include <x509.h>
+
+static int crl_reinit(gnutls_x509_crl_t crl)
+{
+int result;
+
+	if (crl->crl)
+		asn1_delete_structure(&crl->crl);
+
+	result = asn1_create_element(_gnutls_get_pkix(),
+				 "PKIX1.CertificateList",
+				 &crl->crl);
+	if (result != ASN1_SUCCESS) {
+		gnutls_assert();
+		return _gnutls_asn2err(result);
+	}
+	crl->rcache = NULL;
+	crl->rcache_idx = 0;
+	crl->raw_issuer_dn.size = 0;
+
+	return 0;
+}
 
 /**
  * gnutls_x509_crl_init:
- * @crl: The structure to be initialized
+ * @crl: A pointer to the type to be initialized
  *
  * This function will initialize a CRL structure. CRL stands for
  * Certificate Revocation List. A revocation list usually contains
@@ -46,16 +68,16 @@
  **/
 int gnutls_x509_crl_init(gnutls_x509_crl_t * crl)
 {
+	FAIL_IF_LIB_ERROR;
+
 	*crl = gnutls_calloc(1, sizeof(gnutls_x509_crl_int));
 
 	if (*crl) {
-		int result = asn1_create_element(_gnutls_get_pkix(),
-						 "PKIX1.CertificateList",
-						 &(*crl)->crl);
-		if (result != ASN1_SUCCESS) {
+		int result = crl_reinit(*crl);
+		if (result < 0) {
 			gnutls_assert();
 			gnutls_free(*crl);
-			return _gnutls_asn2err(result);
+			return result;
 		}
 		return 0;	/* success */
 	}
@@ -64,7 +86,7 @@ int gnutls_x509_crl_init(gnutls_x509_crl_t * crl)
 
 /**
  * gnutls_x509_crl_deinit:
- * @crl: The structure to be initialized
+ * @crl: The data to be deinitialized
  *
  * This function will deinitialize a CRL structure.
  **/
@@ -75,14 +97,14 @@ void gnutls_x509_crl_deinit(gnutls_x509_crl_t crl)
 
 	if (crl->crl)
 		asn1_delete_structure(&crl->crl);
-	gnutls_free(crl->raw_issuer_dn.data);
+	gnutls_free(crl->der.data);
 
 	gnutls_free(crl);
 }
 
 /**
  * gnutls_x509_crl_import:
- * @crl: The structure to store the parsed CRL.
+ * @crl: The data to store the parsed CRL.
  * @data: The DER or PEM encoded CRL.
  * @format: One of DER or PEM
  *
@@ -99,41 +121,52 @@ gnutls_x509_crl_import(gnutls_x509_crl_t crl,
 		       const gnutls_datum_t * data,
 		       gnutls_x509_crt_fmt_t format)
 {
-	int result = 0, need_free = 0;
-	gnutls_datum_t _data;
-
-	_data.data = data->data;
-	_data.size = data->size;
+	int result = 0;
 
 	if (crl == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
+	_gnutls_free_datum(&crl->der);
+
 	/* If the CRL is in PEM format then decode it
 	 */
 	if (format == GNUTLS_X509_FMT_PEM) {
 		result =
 		    _gnutls_fbase64_decode(PEM_CRL, data->data, data->size,
-					   &_data);
+					   &crl->der);
 
 		if (result < 0) {
 			gnutls_assert();
 			return result;
 		}
-
-		need_free = 1;
+	} else {
+		result = _gnutls_set_datum(&crl->der, data->data, data->size);
+		if (result < 0) {
+			gnutls_assert();
+			return result;
+		}
 	}
 
+	if (crl->expanded) {
+		result = crl_reinit(crl);
+		if (result < 0) {
+			gnutls_assert();
+			goto cleanup;
+		}
+	}
+	crl->expanded = 1;
+
 	result =
-	    asn1_der_decoding(&crl->crl, _data.data, _data.size, NULL);
+	    _asn1_strict_der_decode(&crl->crl, crl->der.data, crl->der.size, NULL);
 	if (result != ASN1_SUCCESS) {
 		result = _gnutls_asn2err(result);
 		gnutls_assert();
 		goto cleanup;
 	}
 
-	result = _gnutls_x509_get_raw_dn2(crl->crl, &_data,
+	result = _gnutls_x509_get_raw_field2(crl->crl, &crl->der,
 					  "tbsCertList.issuer.rdnSequence",
 					  &crl->raw_issuer_dn);
 	if (result < 0) {
@@ -141,22 +174,17 @@ gnutls_x509_crl_import(gnutls_x509_crl_t crl,
 		goto cleanup;
 	}
 
-	if (need_free)
-		_gnutls_free_datum(&_data);
-
 	return 0;
 
       cleanup:
-	if (need_free)
-		_gnutls_free_datum(&_data);
-	_gnutls_free_datum(&crl->raw_issuer_dn);
+	_gnutls_free_datum(&crl->der);
 	return result;
 }
 
 
 /**
  * gnutls_x509_crl_get_issuer_dn:
- * @crl: should contain a gnutls_x509_crl_t structure
+ * @crl: should contain a gnutls_x509_crl_t type
  * @buf: a pointer to a structure to hold the peer's name (may be null)
  * @sizeof_buf: initially holds the size of @buf
  *
@@ -167,6 +195,9 @@ gnutls_x509_crl_import(gnutls_x509_crl_t crl,
  *
  * If buf is %NULL then only the size will be filled.
  *
+ * This function does not output a fully RFC4514 compliant string, if
+ * that is required see gnutls_x509_crl_get_issuer_dn3().
+ *
  * Returns: %GNUTLS_E_SHORT_MEMORY_BUFFER if the provided buffer is
  * not long enough, and in that case the sizeof_buf will be updated
  * with the required size, and 0 on success.
@@ -183,12 +214,12 @@ gnutls_x509_crl_get_issuer_dn(const gnutls_x509_crl_t crl, char *buf,
 
 	return _gnutls_x509_parse_dn(crl->crl,
 				     "tbsCertList.issuer.rdnSequence",
-				     buf, sizeof_buf);
+				     buf, sizeof_buf, GNUTLS_X509_DN_FLAG_COMPAT);
 }
 
 /**
  * gnutls_x509_crl_get_issuer_dn_by_oid:
- * @crl: should contain a gnutls_x509_crl_t structure
+ * @crl: should contain a gnutls_x509_crl_t type
  * @oid: holds an Object Identified in null terminated string
  * @indx: In case multiple same OIDs exist in the RDN, this specifies which to send. Use (0) to get the first one.
  * @raw_flag: If non-zero returns the raw DER data of the DN part.
@@ -214,7 +245,7 @@ gnutls_x509_crl_get_issuer_dn(const gnutls_x509_crl_t crl, char *buf,
  **/
 int
 gnutls_x509_crl_get_issuer_dn_by_oid(gnutls_x509_crl_t crl,
-				     const char *oid, int indx,
+				     const char *oid, unsigned indx,
 				     unsigned int raw_flag, void *buf,
 				     size_t * sizeof_buf)
 {
@@ -238,9 +269,9 @@ gnutls_x509_crl_get_issuer_dn_by_oid(gnutls_x509_crl_t crl,
 
 /**
  * gnutls_x509_crl_get_dn_oid:
- * @crl: should contain a gnutls_x509_crl_t structure
+ * @crl: should contain a gnutls_x509_crl_t type
  * @indx: Specifies which DN OID to send. Use (0) to get the first one.
- * @oid: a pointer to a structure to hold the name (may be null)
+ * @oid: a pointer to store the OID (may be null)
  * @sizeof_oid: initially holds the size of 'oid'
  *
  * This function will extract the requested OID of the name of the CRL
@@ -254,7 +285,7 @@ gnutls_x509_crl_get_issuer_dn_by_oid(gnutls_x509_crl_t crl,
  **/
 int
 gnutls_x509_crl_get_dn_oid(gnutls_x509_crl_t crl,
-			   int indx, void *oid, size_t * sizeof_oid)
+			   unsigned indx, void *oid, size_t * sizeof_oid)
 {
 	if (crl == NULL) {
 		gnutls_assert();
@@ -268,7 +299,7 @@ gnutls_x509_crl_get_dn_oid(gnutls_x509_crl_t crl,
 
 /**
  * gnutls_x509_crl_get_issuer_dn2:
- * @cert: should contain a #gnutls_x509_crt_t structure
+ * @crl: should contain a #gnutls_x509_crl_t type
  * @dn: a pointer to a structure to hold the name
  *
  * This function will allocate buffer and copy the name of the CRL issuer.
@@ -276,8 +307,11 @@ gnutls_x509_crl_get_dn_oid(gnutls_x509_crl_t crl,
  * described in RFC4514. The output string will be ASCII or UTF-8
  * encoded, depending on the certificate data.
  *
+ * This function does not output a fully RFC4514 compliant string, if
+ * that is required see gnutls_x509_crl_get_issuer_dn3().
+ *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
- *   negative error value. and a negative error code on error.
+ *   negative error value.
  *
  * Since: 3.1.10
  **/
@@ -290,12 +324,46 @@ gnutls_x509_crl_get_issuer_dn2(gnutls_x509_crl_t crl, gnutls_datum_t * dn)
 	}
 
 	return _gnutls_x509_get_dn(crl->crl,
-				   "tbsCertList.issuer.rdnSequence", dn);
+				   "tbsCertList.issuer.rdnSequence",
+				   dn, GNUTLS_X509_DN_FLAG_COMPAT);
+}
+
+/**
+ * gnutls_x509_crl_get_issuer_dn3:
+ * @crl: should contain a #gnutls_x509_crl_t type
+ * @dn: a pointer to a structure to hold the name
+ * @flags: zero or %GNUTLS_X509_DN_FLAG_COMPAT
+ *
+ * This function will allocate buffer and copy the name of the CRL issuer.
+ * The name will be in the form "C=xxxx,O=yyyy,CN=zzzz" as
+ * described in RFC4514. The output string will be ASCII or UTF-8
+ * encoded, depending on the certificate data.
+ *
+ * When the flag %GNUTLS_X509_DN_FLAG_COMPAT is specified, the output
+ * format will match the format output by previous to 3.5.6 versions of GnuTLS
+ * which was not not fully RFC4514-compliant.
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
+ *   negative error value.
+ *
+ * Since: 3.5.7
+ **/
+int
+gnutls_x509_crl_get_issuer_dn3(gnutls_x509_crl_t crl, gnutls_datum_t * dn, unsigned flags)
+{
+	if (crl == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	return _gnutls_x509_get_dn(crl->crl,
+				   "tbsCertList.issuer.rdnSequence",
+				   dn, flags);
 }
 
 /**
  * gnutls_x509_crl_get_signature_algorithm:
- * @crl: should contain a #gnutls_x509_crl_t structure
+ * @crl: should contain a #gnutls_x509_crl_t type
  *
  * This function will return a value of the #gnutls_sign_algorithm_t
  * enumeration that is the signature algorithm.
@@ -326,23 +394,63 @@ int gnutls_x509_crl_get_signature_algorithm(gnutls_x509_crl_t crl)
 		return result;
 	}
 
-	result = _gnutls_x509_oid2sign_algorithm((const char *) sa.data);
+	result = gnutls_oid_to_sign((const char *) sa.data);
 
 	_gnutls_free_datum(&sa);
 
 	return result;
 }
 
+/**
+ * gnutls_x509_crl_get_signature_oid:
+ * @crl: should contain a #gnutls_x509_crl_t type
+ * @oid: a pointer to a buffer to hold the OID (may be null)
+ * @oid_size: initially holds the size of @oid
+ *
+ * This function will return the OID of the signature algorithm
+ * that has been used to sign this CRL. This is function
+ * is useful in the case gnutls_x509_crl_get_signature_algorithm()
+ * returned %GNUTLS_SIGN_UNKNOWN.
+ *
+ * Returns: zero or a negative error code on error.
+ *
+ * Since: 3.5.0
+ **/
+int gnutls_x509_crl_get_signature_oid(gnutls_x509_crl_t crl, char *oid, size_t *oid_size)
+{
+	char str[MAX_OID_SIZE];
+	int len, result, ret;
+	gnutls_datum_t out;
+
+	len = sizeof(str);
+	result = asn1_read_value(crl->crl, "signatureAlgorithm.algorithm", str, &len);
+	if (result != ASN1_SUCCESS) {
+		gnutls_assert();
+		return _gnutls_asn2err(result);
+	}
+
+	out.data = (void*)str;
+	out.size = len;
+
+	ret = _gnutls_copy_string(&out, (void*)oid, oid_size);
+	if (ret < 0) {
+		gnutls_assert();
+		return ret;
+	}
+
+	return 0;
+}
+
 /**
  * gnutls_x509_crl_get_signature:
- * @crl: should contain a gnutls_x509_crl_t structure
+ * @crl: should contain a gnutls_x509_crl_t type
  * @sig: a pointer where the signature part will be copied (may be null).
  * @sizeof_sig: initially holds the size of @sig
  *
  * This function will extract the signature field of a CRL.
  *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
- *   negative error value. and a negative error code on error.
+ *   negative error value. 
  **/
 int
 gnutls_x509_crl_get_signature(gnutls_x509_crl_t crl,
@@ -389,7 +497,7 @@ gnutls_x509_crl_get_signature(gnutls_x509_crl_t crl,
 
 /**
  * gnutls_x509_crl_get_version:
- * @crl: should contain a #gnutls_x509_crl_t structure
+ * @crl: should contain a #gnutls_x509_crl_t type
  *
  * This function will return the version of the specified CRL.
  *
@@ -418,7 +526,7 @@ int gnutls_x509_crl_get_version(gnutls_x509_crl_t crl)
 
 /**
  * gnutls_x509_crl_get_this_update:
- * @crl: should contain a #gnutls_x509_crl_t structure
+ * @crl: should contain a #gnutls_x509_crl_t type
  *
  * This function will return the time this CRL was issued.
  *
@@ -437,7 +545,7 @@ time_t gnutls_x509_crl_get_this_update(gnutls_x509_crl_t crl)
 
 /**
  * gnutls_x509_crl_get_next_update:
- * @crl: should contain a #gnutls_x509_crl_t structure
+ * @crl: should contain a #gnutls_x509_crl_t type
  *
  * This function will return the time the next CRL will be issued.
  * This field is optional in a CRL so it might be normal to get an
@@ -458,7 +566,7 @@ time_t gnutls_x509_crl_get_next_update(gnutls_x509_crl_t crl)
 
 /**
  * gnutls_x509_crl_get_crt_count:
- * @crl: should contain a #gnutls_x509_crl_t structure
+ * @crl: should contain a #gnutls_x509_crl_t type
  *
  * This function will return the number of revoked certificates in the
  * given CRL.
@@ -490,7 +598,7 @@ int gnutls_x509_crl_get_crt_count(gnutls_x509_crl_t crl)
 
 /**
  * gnutls_x509_crl_get_crt_serial:
- * @crl: should contain a #gnutls_x509_crl_t structure
+ * @crl: should contain a #gnutls_x509_crl_t type
  * @indx: the index of the certificate to extract (starting from 0)
  * @serial: where the serial number will be copied
  * @serial_size: initially holds the size of serial
@@ -499,11 +607,14 @@ int gnutls_x509_crl_get_crt_count(gnutls_x509_crl_t crl)
  * This function will retrieve the serial number of the specified, by
  * the index, revoked certificate.
  *
+ * Note that this function will have performance issues in large sequences
+ * of revoked certificates. In that case use gnutls_x509_crl_iter_crt_serial().
+ *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
- *   negative error value. and a negative error code on error.
+ *   negative error value.
  **/
 int
-gnutls_x509_crl_get_crt_serial(gnutls_x509_crl_t crl, int indx,
+gnutls_x509_crl_get_crt_serial(gnutls_x509_crl_t crl, unsigned indx,
 			       unsigned char *serial,
 			       size_t * serial_size, time_t * t)
 {
@@ -543,9 +654,109 @@ gnutls_x509_crl_get_crt_serial(gnutls_x509_crl_t crl, int indx,
 	return 0;
 }
 
+/**
+ * gnutls_x509_crl_iter_deinit:
+ * @iter: The iterator to be deinitialized
+ *
+ * This function will deinitialize an iterator type.
+ **/
+void gnutls_x509_crl_iter_deinit(gnutls_x509_crl_iter_t iter)
+{
+	if (!iter)
+		return;
+
+	gnutls_free(iter);
+}
+
+/**
+ * gnutls_x509_crl_iter_crt_serial:
+ * @crl: should contain a #gnutls_x509_crl_t type
+ * @iter: A pointer to an iterator (initially the iterator should be %NULL)
+ * @serial: where the serial number will be copied
+ * @serial_size: initially holds the size of serial
+ * @t: if non null, will hold the time this certificate was revoked
+ *
+ * This function performs the same as gnutls_x509_crl_get_crt_serial(),
+ * but reads sequentially and keeps state in the iterator 
+ * between calls. That allows it to provide better performance in sequences 
+ * with many elements (50000+).
+ *
+ * When past the last element is accessed %GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE
+ * is returned and the iterator is reset.
+ *
+ * After use, the iterator must be deinitialized using gnutls_x509_crl_iter_deinit().
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
+ *   negative error value.
+ **/
+int
+gnutls_x509_crl_iter_crt_serial(gnutls_x509_crl_t crl,
+				gnutls_x509_crl_iter_t *iter,
+				unsigned char *serial,
+				size_t * serial_size, time_t * t)
+{
+
+	int result, _serial_size;
+	char serial_name[ASN1_MAX_NAME_SIZE];
+	char date_name[ASN1_MAX_NAME_SIZE];
+
+	if (crl == NULL || iter == NULL) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_REQUEST;
+	}
+
+	if (*iter == NULL) {
+		*iter = gnutls_calloc(1, sizeof(struct gnutls_x509_crl_iter));
+		if (*iter == NULL)
+			return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+	}
+
+	if ((*iter)->rcache == NULL) {
+		(*iter)->rcache = asn1_find_node (crl->crl, "tbsCertList.revokedCertificates.?1");
+		(*iter)->rcache_idx = 1;
+	} else {
+		snprintf(serial_name, sizeof(serial_name),
+			 "?%d", (*iter)->rcache_idx);
+		(*iter)->rcache = asn1_find_node ((*iter)->rcache, serial_name);
+	}
+	if ((*iter)->rcache == NULL) {
+		/* reset */
+		(*iter)->rcache = NULL;
+		return gnutls_assert_val(GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE);
+	}
+
+	snprintf(serial_name, sizeof(serial_name),
+		 "?%d.userCertificate", (*iter)->rcache_idx);
+
+	_serial_size = *serial_size;
+	result =
+	    asn1_read_value((*iter)->rcache, serial_name, serial, &_serial_size);
+
+	*serial_size = _serial_size;
+	if (result != ASN1_SUCCESS) {
+		gnutls_assert();
+		if (result == ASN1_ELEMENT_NOT_FOUND) {
+			/* reset */
+			(*iter)->rcache = NULL;
+			return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
+		}
+		return _gnutls_asn2err(result);
+	}
+
+	if (t) {
+		snprintf(date_name, sizeof(date_name),
+			 "?%d.revocationDate", (*iter)->rcache_idx);
+		*t = _gnutls_x509_get_time((*iter)->rcache, date_name, 0);
+	}
+
+	(*iter)->rcache_idx++;
+
+	return 0;
+}
+
 /**
  * gnutls_x509_crl_get_raw_issuer_dn:
- * @crl: should contain a gnutls_x509_crl_t structure
+ * @crl: should contain a gnutls_x509_crl_t type
  * @dn: will hold the starting point of the DN
  *
  * This function will return a pointer to the DER encoded DN structure
@@ -559,8 +770,12 @@ int
 gnutls_x509_crl_get_raw_issuer_dn(gnutls_x509_crl_t crl,
 				  gnutls_datum_t * dn)
 {
-	return _gnutls_set_datum(dn, crl->raw_issuer_dn.data,
-				 crl->raw_issuer_dn.size);
+	if (crl->raw_issuer_dn.size != 0) {
+		return _gnutls_set_datum(dn, crl->raw_issuer_dn.data,
+					 crl->raw_issuer_dn.size);
+	} else {
+		return _gnutls_x509_get_raw_field(crl->crl, "tbsCertList.issuer.rdnSequence", dn);
+	}
 }
 
 /**
@@ -580,7 +795,7 @@ gnutls_x509_crl_get_raw_issuer_dn(gnutls_x509_crl_t crl,
  * of "BEGIN X509 CRL".
  *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
- *   negative error value. and a negative error code on failure.
+ *   negative error value.
  **/
 int
 gnutls_x509_crl_export(gnutls_x509_crl_t crl,
@@ -610,7 +825,7 @@ gnutls_x509_crl_export(gnutls_x509_crl_t crl,
  * of "BEGIN X509 CRL".
  *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
- *   negative error value. and a negative error code on failure.
+ *   negative error value.
  *
  * Since 3.1.3
  **/
@@ -627,9 +842,9 @@ gnutls_x509_crl_export2(gnutls_x509_crl_t crl,
 }
 
 /*-
- * _gnutls_x509_crl_cpy - This function copies a gnutls_x509_crl_t structure
- * @dest: The structure where to copy
- * @src: The structure to be copied
+ * _gnutls_x509_crl_cpy - This function copies a gnutls_x509_crl_t type
+ * @dest: The data where to copy
+ * @src: The data to be copied
  *
  * This function will copy an X.509 certificate structure.
  *
@@ -691,7 +906,7 @@ _get_authority_key_id(gnutls_x509_crl_t cert, ASN1_TYPE * c2,
 		return _gnutls_asn2err(ret);
 	}
 
-	ret = asn1_der_decoding(c2, id.data, id.size, NULL);
+	ret = _asn1_strict_der_decode(c2, id.data, id.size, NULL);
 	_gnutls_free_datum(&id);
 
 	if (ret != ASN1_SUCCESS) {
@@ -705,7 +920,7 @@ _get_authority_key_id(gnutls_x509_crl_t cert, ASN1_TYPE * c2,
 
 /**
  * gnutls_x509_crl_get_authority_key_gn_serial:
- * @crl: should contain a #gnutls_x509_crl_t structure
+ * @crl: should contain a #gnutls_x509_crl_t type
  * @seq: specifies the sequence number of the alt name (0 for the first one, 1 for the second etc.)
  * @alt: is the place where the alternative name will be copied to
  * @alt_size: holds the size of alt.
@@ -777,7 +992,7 @@ gnutls_x509_crl_get_authority_key_gn_serial(gnutls_x509_crl_t crl,
 
 /**
  * gnutls_x509_crl_get_authority_key_id:
- * @crl: should contain a #gnutls_x509_crl_t structure
+ * @crl: should contain a #gnutls_x509_crl_t type
  * @id: The place where the identifier will be copied
  * @id_size: Holds the size of the result field.
  * @critical: will be non-zero if the extension is marked as critical
@@ -829,7 +1044,7 @@ gnutls_x509_crl_get_authority_key_id(gnutls_x509_crl_t crl, void *id,
 
 /**
  * gnutls_x509_crl_get_number:
- * @crl: should contain a #gnutls_x509_crl_t structure
+ * @crl: should contain a #gnutls_x509_crl_t type
  * @ret: The place where the number will be copied
  * @ret_size: Holds the size of the result field.
  * @critical: will be non-zero if the extension is marked as critical
@@ -855,7 +1070,6 @@ gnutls_x509_crl_get_number(gnutls_x509_crl_t crl, void *ret,
 		return GNUTLS_E_INVALID_REQUEST;
 	}
 
-
 	if (ret)
 		memset(ret, 0, *ret_size);
 	else
@@ -888,9 +1102,9 @@ gnutls_x509_crl_get_number(gnutls_x509_crl_t crl, void *ret,
 
 /**
  * gnutls_x509_crl_get_extension_oid:
- * @crl: should contain a #gnutls_x509_crl_t structure
+ * @crl: should contain a #gnutls_x509_crl_t type
  * @indx: Specifies which extension OID to send, use (0) to get the first one.
- * @oid: a pointer to a structure to hold the OID (may be null)
+ * @oid: a pointer to store the OID (may be null)
  * @sizeof_oid: initially holds the size of @oid
  *
  * This function will return the requested extension OID in the CRL.
@@ -905,7 +1119,7 @@ gnutls_x509_crl_get_number(gnutls_x509_crl_t crl, void *ret,
  * Since: 2.8.0
  **/
 int
-gnutls_x509_crl_get_extension_oid(gnutls_x509_crl_t crl, int indx,
+gnutls_x509_crl_get_extension_oid(gnutls_x509_crl_t crl, unsigned indx,
 				  void *oid, size_t * sizeof_oid)
 {
 	int result;
@@ -927,9 +1141,9 @@ gnutls_x509_crl_get_extension_oid(gnutls_x509_crl_t crl, int indx,
 
 /**
  * gnutls_x509_crl_get_extension_info:
- * @crl: should contain a #gnutls_x509_crl_t structure
+ * @crl: should contain a #gnutls_x509_crl_t type
  * @indx: Specifies which extension OID to send, use (0) to get the first one.
- * @oid: a pointer to a structure to hold the OID
+ * @oid: a pointer to store the OID
  * @sizeof_oid: initially holds the maximum size of @oid, on return
  *   holds actual size of @oid.
  * @critical: output variable with critical flag, may be NULL.
@@ -951,7 +1165,7 @@ gnutls_x509_crl_get_extension_oid(gnutls_x509_crl_t crl, int indx,
  * Since: 2.8.0
  **/
 int
-gnutls_x509_crl_get_extension_info(gnutls_x509_crl_t crl, int indx,
+gnutls_x509_crl_get_extension_info(gnutls_x509_crl_t crl, unsigned indx,
 				   void *oid, size_t * sizeof_oid,
 				   unsigned int *critical)
 {
@@ -1001,7 +1215,7 @@ gnutls_x509_crl_get_extension_info(gnutls_x509_crl_t crl, int indx,
 
 /**
  * gnutls_x509_crl_get_extension_data:
- * @crl: should contain a #gnutls_x509_crl_t structure
+ * @crl: should contain a #gnutls_x509_crl_t type
  * @indx: Specifies which extension OID to send. Use (0) to get the first one.
  * @data: a pointer to a structure to hold the data (may be null)
  * @sizeof_data: initially holds the size of @oid
@@ -1023,7 +1237,7 @@ gnutls_x509_crl_get_extension_info(gnutls_x509_crl_t crl, int indx,
  * Since: 2.8.0
  **/
 int
-gnutls_x509_crl_get_extension_data(gnutls_x509_crl_t crl, int indx,
+gnutls_x509_crl_get_extension_data(gnutls_x509_crl_t crl, unsigned indx,
 				   void *data, size_t * sizeof_data)
 {
 	int result, len;
@@ -1053,7 +1267,7 @@ gnutls_x509_crl_get_extension_data(gnutls_x509_crl_t crl, int indx,
 
 /**
  * gnutls_x509_crl_list_import2:
- * @crls: The structures to store the parsed crl list. Must not be initialized.
+ * @crls: Will contain the parsed crl list.
  * @size: It will contain the size of the list.
  * @data: The PEM encoded CRL.
  * @format: One of DER or PEM.
@@ -1088,7 +1302,7 @@ gnutls_x509_crl_list_import2(gnutls_x509_crl_t ** crls,
 
 	ret =
 	    gnutls_x509_crl_list_import(*crls, &init, data, format,
-					GNUTLS_X509_CRT_LIST_IMPORT_FAIL_IF_EXCEED);
+					flags | GNUTLS_X509_CRT_LIST_IMPORT_FAIL_IF_EXCEED);
 	if (ret == GNUTLS_E_SHORT_MEMORY_BUFFER) {
 		*crls =
 		    gnutls_realloc_fast(*crls,
@@ -1115,7 +1329,7 @@ gnutls_x509_crl_list_import2(gnutls_x509_crl_t ** crls,
 
 /**
  * gnutls_x509_crl_list_import:
- * @crls: The structures to store the parsed CRLs. Must not be initialized.
+ * @crls: Indicates where the parsed CRLs will be copied to. Must not be initialized.
  * @crl_max: Initially must hold the maximum number of crls. It will be updated with the number of crls available.
  * @data: The PEM encoded CRLs
  * @format: One of DER or PEM.
