@@ -24,10 +24,10 @@
  */
 
 #include "gnutls_int.h"
-#include "gnutls_errors.h"
-#include <gnutls_datum.h>
+#include "errors.h"
+#include <datum.h>
 #include <x509_b64.h>
-#include <base64.h>
+#include <nettle/base64.h>
 
 #define INCR(what, size, max_len) \
 	do { \
@@ -48,7 +48,7 @@ _gnutls_fbase64_encode(const char *msg, const uint8_t * data,
 {
 	int tmp;
 	unsigned int i;
-	char tmpres[66];
+	uint8_t tmpres[66];
 	uint8_t *ptr;
 	char top[80];
 	char bottom[80];
@@ -91,9 +91,11 @@ _gnutls_fbase64_encode(const char *msg, const uint8_t * data,
 		else
 			tmp = 48;
 
-		base64_encode((void *) &data[i], tmp, tmpres,
-			      sizeof(tmpres));
-		size = strlen(tmpres);
+		size = BASE64_ENCODE_RAW_LENGTH(tmp);
+		if (sizeof(tmpres) < size)
+			return gnutls_assert_val(GNUTLS_E_BASE64_ENCODING_ERROR);
+
+		base64_encode_raw(tmpres, tmp, &data[i]);
 
 		INCR(bytes, size + 1, max);
 		ptr = &result->data[pos];
@@ -168,8 +170,14 @@ gnutls_pem_base64_encode(const char *msg, const gnutls_datum_t * data,
  *
  * You should use gnutls_free() to free the returned data.
  *
+ * Note, that prior to GnuTLS 3.4.0 this function was available
+ * under the name gnutls_pem_base64_encode_alloc(). There is
+ * compatibility macro pointing to this function.
+ *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise
  *   an error code is returned.
+ *
+ * Since: 3.4.0
  **/
 int
 gnutls_pem_base64_encode2(const char *msg,
@@ -222,11 +230,10 @@ int
 _gnutls_base64_decode(const uint8_t * data, size_t data_size,
 		      gnutls_datum_t * result)
 {
-	unsigned int i;
-	int pos, tmp, est, ret;
-	uint8_t tmpres[48];
-	size_t tmpres_size, decode_size;
+	int ret;
+	size_t size;
 	gnutls_datum_t pdata;
+	struct base64_decode_ctx ctx;
 
 	ret = cpydata(data, data_size, &pdata);
 	if (ret < 0) {
@@ -234,37 +241,31 @@ _gnutls_base64_decode(const uint8_t * data, size_t data_size,
 		return ret;
 	}
 
-	est = ((data_size * 3) / 4) + 1;
+	base64_decode_init(&ctx);
 
-	result->data = gnutls_malloc(est);
+	size = BASE64_DECODE_LENGTH(data_size);
+
+	result->data = gnutls_malloc(size);
 	if (result->data == NULL)
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 
-	pos = 0;
-	for (i = 0; i < pdata.size; i += 64) {
-		if (pdata.size - i < 64)
-			decode_size = pdata.size - i;
-		else
-			decode_size = 64;
-
-		tmpres_size = sizeof(tmpres);
-		tmp =
-		    base64_decode((void *) &pdata.data[i], decode_size,
-				  (void *) tmpres, &tmpres_size);
-		if (tmp == 0) {
-			gnutls_assert();
-			gnutls_free(result->data);
-			result->data = NULL;
-			ret = GNUTLS_E_PARSING_ERROR;
-			goto cleanup;
-		}
-		memcpy(&result->data[pos], tmpres, tmpres_size);
-		pos += tmpres_size;
+	ret = base64_decode_update(&ctx, &size, result->data,
+				   pdata.size, pdata.data); 
+	if (ret == 0) {
+		gnutls_assert();
+		gnutls_free(result->data);
+		result->data = NULL;
+		ret = GNUTLS_E_PARSING_ERROR;
+		goto cleanup;
 	}
 
-	result->size = pos;
+	ret = base64_decode_final(&ctx);
+	if (ret != 1)
+		return gnutls_assert_val(GNUTLS_E_PARSING_ERROR);
 
-	ret = pos;
+	result->size = size;
+
+	ret = size;
 
       cleanup:
 	gnutls_free(pdata.data);
@@ -301,7 +302,7 @@ _gnutls_fbase64_decode(const char *header, const uint8_t * data,
 		return GNUTLS_E_BASE64_UNEXPECTED_HEADER_ERROR;
 	}
 
-	data_size -= (unsigned long int) rdata - (unsigned long int) data;
+	data_size -= MEMSUB(rdata, data);
 
 	if (data_size < 4 + strlen(bottom)) {
 		gnutls_assert();
@@ -318,7 +319,7 @@ _gnutls_fbase64_decode(const char *header, const uint8_t * data,
 		return GNUTLS_E_BASE64_DECODING_ERROR;
 	}
 	data_size -= strlen(ENDSTR);
-	data_size -= (unsigned long int) kdata - (unsigned long int) rdata;
+	data_size -= MEMSUB(kdata, rdata);
 
 	rdata = kdata + strlen(ENDSTR);
 
@@ -332,7 +333,7 @@ _gnutls_fbase64_decode(const char *header, const uint8_t * data,
 
 	/* position of kdata is before the ----END--- footer 
 	 */
-	rdata_size = (unsigned long int) kdata - (unsigned long int) rdata;
+	rdata_size = MEMSUB(kdata, rdata);
 
 	if (rdata_size < 4) {
 		gnutls_assert();
@@ -404,8 +405,14 @@ gnutls_pem_base64_decode(const char *header,
  *
  * You should use gnutls_free() to free the returned data.
  *
+ * Note, that prior to GnuTLS 3.4.0 this function was available
+ * under the name gnutls_pem_base64_decode_alloc(). There is
+ * compatibility macro pointing to this function.
+ *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise
  *   an error code is returned.
+ *
+ * Since: 3.4.0
  **/
 int
 gnutls_pem_base64_decode2(const char *header,
