@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include "common.h"
 
 #include <gnutls/gnutls.h>
 #include <gnutls/ocsp.h>
@@ -37,7 +38,7 @@
 #include <ocsptool-common.h>
 
 #define MAX_BUF 4*1024
-#define HEADER_PATTERN "POST /%s HTTP/1.1\r\n" \
+#define HEADER_PATTERN "POST /%s HTTP/1.0\r\n" \
   "Host: %s\r\n" \
   "Accept: */*\r\n" \
   "Content-Type: application/ocsp-request\r\n" \
@@ -114,7 +115,7 @@ _generate_request(gnutls_x509_crt_t cert, gnutls_x509_crt_t issuer,
 	return;
 }
 
-static size_t get_data(void *buffer, size_t size, size_t nmemb,
+static size_t get_data(void *buf, size_t size, size_t nmemb,
 		       void *userp)
 {
 	gnutls_datum_t *ud = userp;
@@ -127,7 +128,7 @@ static size_t get_data(void *buffer, size_t size, size_t nmemb,
 		exit(1);
 	}
 
-	memcpy(&ud->data[ud->size], buffer, size);
+	memcpy(&ud->data[ud->size], buf, size);
 	ud->size += size;
 
 	return size;
@@ -147,6 +148,7 @@ int send_ocsp_request(const char *server,
 	unsigned char *p;
 	const char *hostname;
 	const char *path = "";
+	unsigned i;
 	unsigned int headers_size = 0, port;
 	socket_st hd;
 
@@ -156,20 +158,28 @@ int send_ocsp_request(const char *server,
 		/* try to read URL from issuer certificate */
 		gnutls_datum_t data;
 
-		ret = gnutls_x509_crt_get_authority_info_access(cert, 0,
-								GNUTLS_IA_OCSP_URI,
-								&data,
-								NULL);
+		i = 0;
+		do {
+			ret = gnutls_x509_crt_get_authority_info_access(cert, i++,
+									GNUTLS_IA_OCSP_URI,
+									&data,
+									NULL);
+		} while(ret == GNUTLS_E_UNKNOWN_ALGORITHM);
+
+		if (ret < 0) {
+			i = 0;
+			do {
+				ret =
+				    gnutls_x509_crt_get_authority_info_access
+				    (issuer, i++, GNUTLS_IA_OCSP_URI, &data, NULL);
+			} while(ret == GNUTLS_E_UNKNOWN_ALGORITHM);
+		}
 
-		if (ret < 0)
-			ret =
-			    gnutls_x509_crt_get_authority_info_access
-			    (issuer, 0, GNUTLS_IA_OCSP_URI, &data, NULL);
 		if (ret < 0) {
 			fprintf(stderr,
-				"Cannot find URL from issuer: %s\n",
+				"*** Cannot find OCSP server URI in certificate: %s\n",
 				gnutls_strerror(ret));
-			return -1;
+			return ret;
 		}
 
 		url = malloc(data.size + 1);
@@ -195,10 +205,11 @@ int send_ocsp_request(const char *server,
 		 (unsigned int) req.size);
 	headers_size = strlen(headers);
 
-	socket_open(&hd, hostname, service, 0);
+	socket_open(&hd, hostname, service, NULL, SOCKET_FLAG_RAW|SOCKET_FLAG_SKIP_INIT, CONNECT_MSG, NULL);
 
 	socket_send(&hd, headers, headers_size);
 	socket_send(&hd, req.data, req.size);
+	gnutls_free(req.data);
 
 	do {
 		ret = socket_recv(&hd, buffer, sizeof(buffer));
@@ -211,7 +222,7 @@ int send_ocsp_request(const char *server,
 		return -1;
 	}
 
-	socket_bye(&hd);
+	socket_bye(&hd, 0);
 
 	p = memmem(ud.data, ud.size, "\r\n\r\n", 4);
 	if (p == NULL) {
@@ -228,6 +239,8 @@ int send_ocsp_request(const char *server,
 	memcpy(resp_data->data, p, resp_data->size);
 
 	free(ud.data);
+	if (url != server)
+		free(url);
 
 	return 0;
 }
@@ -305,7 +318,7 @@ void print_ocsp_verify_res(unsigned int output)
 int
 check_ocsp_response(gnutls_x509_crt_t cert,
 		    gnutls_x509_crt_t issuer, gnutls_datum_t * data,
-		    gnutls_datum_t * nonce)
+		    gnutls_datum_t * nonce, int verbose)
 {
 	gnutls_ocsp_resp_t resp;
 	int ret;
@@ -330,8 +343,13 @@ check_ocsp_response(gnutls_x509_crt_t cert,
 
 	ret = gnutls_ocsp_resp_check_crt(resp, 0, cert);
 	if (ret < 0) {
-		printf
-		    ("*** Got OCSP response on an unrelated certificate (ignoring)\n");
+		if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
+			printf
+			    ("*** Got OCSP response with no data (ignoring)\n");
+		} else {
+			printf
+			    ("*** Got OCSP response on an unrelated certificate (ignoring)\n");
+		}
 		ret = -1;
 		goto cleanup;
 	}
@@ -391,11 +409,12 @@ check_ocsp_response(gnutls_x509_crt_t cert,
 	}
 
 	if (nonce) {
-	        gnutls_datum_t rnonce;
+		gnutls_datum_t rnonce;
 
 		ret = gnutls_ocsp_resp_get_nonce(resp, NULL, &rnonce);
 		if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
-			fprintf(stderr, "*** The OCSP reply did not include the requested nonce.\n");
+			if (verbose)
+				fprintf(stderr, "*** The OCSP reply did not include the requested nonce.\n");
 			goto finish_ok;
 		}
 
@@ -411,7 +430,7 @@ check_ocsp_response(gnutls_x509_crt_t cert,
 			exit(1);
 		}
 
-	        gnutls_free(rnonce.data);
+		gnutls_free(rnonce.data);
 	}
 
  finish_ok:
