@@ -1,5 +1,7 @@
 /*
- * Copyright (C) 2012 Sean Buckheister
+ * Copyright (C) 2012-2016 Sean Buckheister
+ * Copyright (C) 2016 Nikos Mavrogiannopoulos
+ * Copyright (C) 2016 Red Hat, Inc.
  *
  * This file is part of GnuTLS.
  *
@@ -23,25 +25,27 @@
  *
  * **** Available parameters ****
  *
- *	-nb                 enable nonblocking operations on sessions
- *	-batch              read test identifiers from stdin and run them
- *	-d                  increase debug level by one
- *	-d <n>              set debug level to <n>
- *	-die                don't start new tests after the first detected failure
- *	-timeout <n>        set handshake timeout to <n> seconds. Tests that don't make progress
- *	                    within twice this time will be forcibly killed. (default: 120)
+ *	-nb		 enable nonblocking operations on sessions
+ *	-batch	      read test identifiers from stdin and run them
+ *	-d		  increase debug level by one
+ *	-r		  replay messages (very crude replay mechanism)
+ *	-d <n>	      set debug level to <n>
+ *	-die		don't start new tests after the first detected failure
+ *	-timeout <n>	set handshake timeout to <n> seconds. Tests that don't make progress
+ *			    within twice this time will be forcibly killed. (default: 120)
  *	-retransmit <n>     set retransmit timeout to <n> milliseconds (default: 100)
- *	-j <n>              run up to <n> tests in parallel
- *	-full               use full handshake with mutual certificate authentication
+ *	-j <n>	      run up to <n> tests in parallel
+ *	-full	       use full handshake with mutual certificate authentication
+ *	-resume	     use resumed handshake
  *	-shello <perm>      run only one test, with the server hello flight permuted as <perm>
  *	-sfinished <perm>   run only one test, with the server finished flight permuted as <perm>
  *	-cfinished <perm>   run only one test, with the client finished flight permuted as <perm>
  *	<packet name>       run only one test, drop <packet name> three times
- *	                    valid values for <packet name> are:
- *	                        SHello, SCertificate, SKeyExchange, SCertificateRequest, SHelloDone,
- *	                        CCertificate, CKeyExchange, CCertificateVerify, CChangeCipherSpec,
- *	                        CFinished, SChangeCipherSpec, SFinished
- *	                    using *Certificate* without -full will yield unexpected results
+ *			    valid values for <packet name> are:
+ *				SHello, SCertificate, SKeyExchange, SCertificateRequest, SHelloDone,
+ *				CCertificate, CKeyExchange, CCertificateVerify, CChangeCipherSpec,
+ *				CFinished, SChangeCipherSpec, SFinished
+ *			    using *Certificate* without -full will yield unexpected results
  *
  * 
  * **** Permutation handling ****
@@ -51,6 +55,8 @@
  * For example, -full -shello 42130 will transmit server hello flight packets in the order
  * SHelloDone, SKeyExchange, SCertificate, SCertificateRequest, SHello
  *
+ * When -resume is specified the -sfinished flight length is 3 (same as shello), cfinished is 2.
+ * The -resume option has to be combined with sfinished or cfinished.
  *
  * **** Output format ****
  *
@@ -101,11 +107,15 @@
 
 // {{{ types
 
+#define log(fmt, ...) \
+	if (debug) fprintf(stdout, "%i %s| "fmt, run_id, role_name, ##__VA_ARGS__)
+
 typedef struct {
 	int count;
 } filter_packet_state_t;
 
 typedef struct {
+	const char *name;
 	gnutls_datum_t packets[5];
 	int *order;
 	int count;
@@ -134,112 +144,58 @@ static int permutations3[6][3]
 static const char *permutation_names3[]
 = { "012", "021", "102", "120", "201", "210", 0 };
 
-static int permutations5[120][5]
-= { {0, 1, 2, 3, 4}, {0, 2, 1, 3, 4}, {1, 0, 2, 3, 4}, {1, 2, 0, 3, 4}, {2,
-									 0,
-									 1,
-									 3,
-									 4},
-    {2, 1, 0, 3, 4},
-{0, 1, 3, 2, 4}, {0, 2, 3, 1, 4}, {1, 0, 3, 2, 4}, {1, 2, 3, 0, 4}, {2, 0,
-								     3, 1,
-								     4},
-    {2, 1, 3, 0, 4},
-{0, 3, 1, 2, 4}, {0, 3, 2, 1, 4}, {1, 3, 0, 2, 4}, {1, 3, 2, 0, 4}, {2, 3,
-								     0, 1,
-								     4},
-    {2, 3, 1, 0, 4},
-{3, 0, 1, 2, 4}, {3, 0, 2, 1, 4}, {3, 1, 0, 2, 4}, {3, 1, 2, 0, 4}, {3, 2,
-								     0, 1,
-								     4},
-    {3, 2, 1, 0, 4},
-{0, 1, 2, 4, 3}, {0, 2, 1, 4, 3}, {1, 0, 2, 4, 3}, {1, 2, 0, 4, 3}, {2, 0,
-								     1, 4,
-								     3},
-    {2, 1, 0, 4, 3},
-{0, 1, 3, 4, 2}, {0, 2, 3, 4, 1}, {1, 0, 3, 4, 2}, {1, 2, 3, 4, 0}, {2, 0,
-								     3, 4,
-								     1},
-    {2, 1, 3, 4, 0},
-{0, 3, 1, 4, 2}, {0, 3, 2, 4, 1}, {1, 3, 0, 4, 2}, {1, 3, 2, 4, 0}, {2, 3,
-								     0, 4,
-								     1},
-    {2, 3, 1, 4, 0},
-{3, 0, 1, 4, 2}, {3, 0, 2, 4, 1}, {3, 1, 0, 4, 2}, {3, 1, 2, 4, 0}, {3, 2,
-								     0, 4,
-								     1},
-    {3, 2, 1, 4, 0},
-{0, 1, 4, 2, 3}, {0, 2, 4, 1, 3}, {1, 0, 4, 2, 3}, {1, 2, 4, 0, 3}, {2, 0,
-								     4, 1,
-								     3},
-    {2, 1, 4, 0, 3},
-{0, 1, 4, 3, 2}, {0, 2, 4, 3, 1}, {1, 0, 4, 3, 2}, {1, 2, 4, 3, 0}, {2, 0,
-								     4, 3,
-								     1},
-    {2, 1, 4, 3, 0},
-{0, 3, 4, 1, 2}, {0, 3, 4, 2, 1}, {1, 3, 4, 0, 2}, {1, 3, 4, 2, 0}, {2, 3,
-								     4, 0,
-								     1},
-    {2, 3, 4, 1, 0},
-{3, 0, 4, 1, 2}, {3, 0, 4, 2, 1}, {3, 1, 4, 0, 2}, {3, 1, 4, 2, 0}, {3, 2,
-								     4, 0,
-								     1},
-    {3, 2, 4, 1, 0},
-{0, 4, 1, 2, 3}, {0, 4, 2, 1, 3}, {1, 4, 0, 2, 3}, {1, 4, 2, 0, 3}, {2, 4,
-								     0, 1,
-								     3},
-    {2, 4, 1, 0, 3},
-{0, 4, 1, 3, 2}, {0, 4, 2, 3, 1}, {1, 4, 0, 3, 2}, {1, 4, 2, 3, 0}, {2, 4,
-								     0, 3,
-								     1},
-    {2, 4, 1, 3, 0},
-{0, 4, 3, 1, 2}, {0, 4, 3, 2, 1}, {1, 4, 3, 0, 2}, {1, 4, 3, 2, 0}, {2, 4,
-								     3, 0,
-								     1},
-    {2, 4, 3, 1, 0},
-{3, 4, 0, 1, 2}, {3, 4, 0, 2, 1}, {3, 4, 1, 0, 2}, {3, 4, 1, 2, 0}, {3, 4,
-								     2, 0,
-								     1},
-    {3, 4, 2, 1, 0},
-{4, 0, 1, 2, 3}, {4, 0, 2, 1, 3}, {4, 1, 0, 2, 3}, {4, 1, 2, 0, 3}, {4, 2,
-								     0, 1,
-								     3},
-    {4, 2, 1, 0, 3},
-{4, 0, 1, 3, 2}, {4, 0, 2, 3, 1}, {4, 1, 0, 3, 2}, {4, 1, 2, 3, 0}, {4, 2,
-								     0, 3,
-								     1},
-    {4, 2, 1, 3, 0},
-{4, 0, 3, 1, 2}, {4, 0, 3, 2, 1}, {4, 1, 3, 0, 2}, {4, 1, 3, 2, 0}, {4, 2,
-								     3, 0,
-								     1},
-    {4, 2, 3, 1, 0},
-{4, 3, 0, 1, 2}, {4, 3, 0, 2, 1}, {4, 3, 1, 0, 2}, {4, 3, 1, 2, 0}, {4, 3,
-								     2, 0,
-								     1},
-    {4, 3, 2, 1, 0}
+static int permutations5[120][5] = { 
+	{0, 1, 2, 3, 4}, {0, 2, 1, 3, 4}, {1, 0, 2, 3, 4}, {1, 2, 0, 3, 4},
+	{2, 0, 1, 3, 4}, {2, 1, 0, 3, 4}, {0, 1, 3, 2, 4}, {0, 2, 3, 1, 4},
+	{1, 0, 3, 2, 4}, {1, 2, 3, 0, 4}, {2, 0, 3, 1, 4}, {2, 1, 3, 0, 4},
+	{0, 3, 1, 2, 4}, {0, 3, 2, 1, 4}, {1, 3, 0, 2, 4}, {1, 3, 2, 0, 4},
+	{2, 3, 0, 1, 4}, {2, 3, 1, 0, 4}, {3, 0, 1, 2, 4}, {3, 0, 2, 1, 4},
+	{3, 1, 0, 2, 4}, {3, 1, 2, 0, 4}, {3, 2, 0, 1, 4}, {3, 2, 1, 0, 4},
+	{0, 1, 2, 4, 3}, {0, 2, 1, 4, 3}, {1, 0, 2, 4, 3}, {1, 2, 0, 4, 3},
+	{2, 0, 1, 4, 3}, {2, 1, 0, 4, 3}, {0, 1, 3, 4, 2}, {0, 2, 3, 4, 1},
+	{1, 0, 3, 4, 2}, {1, 2, 3, 4, 0}, {2, 0, 3, 4, 1}, {2, 1, 3, 4, 0},
+	{0, 3, 1, 4, 2}, {0, 3, 2, 4, 1}, {1, 3, 0, 4, 2}, {1, 3, 2, 4, 0},
+	{2, 3, 0, 4, 1}, {2, 3, 1, 4, 0}, {3, 0, 1, 4, 2}, {3, 0, 2, 4, 1},
+	{3, 1, 0, 4, 2}, {3, 1, 2, 4, 0}, {3, 2, 0, 4, 1}, {3, 2, 1, 4, 0},
+	{0, 1, 4, 2, 3}, {0, 2, 4, 1, 3}, {1, 0, 4, 2, 3}, {1, 2, 4, 0, 3},
+	{2, 0, 4, 1, 3}, {2, 1, 4, 0, 3}, {0, 1, 4, 3, 2}, {0, 2, 4, 3, 1},
+	{1, 0, 4, 3, 2}, {1, 2, 4, 3, 0}, {2, 0, 4, 3, 1}, {2, 1, 4, 3, 0},
+	{0, 3, 4, 1, 2}, {0, 3, 4, 2, 1}, {1, 3, 4, 0, 2}, {1, 3, 4, 2, 0},
+	{2, 3, 4, 0, 1}, {2, 3, 4, 1, 0}, {3, 0, 4, 1, 2}, {3, 0, 4, 2, 1},
+	{3, 1, 4, 0, 2}, {3, 1, 4, 2, 0}, {3, 2, 4, 0, 1}, {3, 2, 4, 1, 0},
+	{0, 4, 1, 2, 3}, {0, 4, 2, 1, 3}, {1, 4, 0, 2, 3}, {1, 4, 2, 0, 3},
+	{2, 4, 0, 1, 3}, {2, 4, 1, 0, 3}, {0, 4, 1, 3, 2}, {0, 4, 2, 3, 1},
+	{1, 4, 0, 3, 2}, {1, 4, 2, 3, 0}, {2, 4, 0, 3, 1}, {2, 4, 1, 3, 0},
+	{0, 4, 3, 1, 2}, {0, 4, 3, 2, 1}, {1, 4, 3, 0, 2}, {1, 4, 3, 2, 0},
+	{2, 4, 3, 0, 1}, {2, 4, 3, 1, 0}, {3, 4, 0, 1, 2}, {3, 4, 0, 2, 1},
+	{3, 4, 1, 0, 2}, {3, 4, 1, 2, 0}, {3, 4, 2, 0, 1}, {3, 4, 2, 1, 0},
+	{4, 0, 1, 2, 3}, {4, 0, 2, 1, 3}, {4, 1, 0, 2, 3}, {4, 1, 2, 0, 3},
+	{4, 2, 0, 1, 3}, {4, 2, 1, 0, 3}, {4, 0, 1, 3, 2}, {4, 0, 2, 3, 1},
+	{4, 1, 0, 3, 2}, {4, 1, 2, 3, 0}, {4, 2, 0, 3, 1}, {4, 2, 1, 3, 0},
+	{4, 0, 3, 1, 2}, {4, 0, 3, 2, 1}, {4, 1, 3, 0, 2}, {4, 1, 3, 2, 0},
+	{4, 2, 3, 0, 1}, {4, 2, 3, 1, 0}, {4, 3, 0, 1, 2}, {4, 3, 0, 2, 1},
+	{4, 3, 1, 0, 2}, {4, 3, 1, 2, 0}, {4, 3, 2, 0, 1}, {4, 3, 2, 1, 0}
 };
 
 static const char *permutation_names5[]
     = { "01234", "02134", "10234", "12034", "20134", "21034", "01324",
-	    "02314", "10324", "12304", "20314", "21304",
-	"03124", "03214", "13024", "13204", "23014", "23104", "30124",
-	    "30214", "31024", "31204", "32014", "32104",
-	"01243", "02143", "10243", "12043", "20143", "21043", "01342",
-	    "02341", "10342", "12340", "20341", "21340",
-	"03142", "03241", "13042", "13240", "23041", "23140", "30142",
-	    "30241", "31042", "31240", "32041", "32140",
-	"01423", "02413", "10423", "12403", "20413", "21403", "01432",
-	    "02431", "10432", "12430", "20431", "21430",
-	"03412", "03421", "13402", "13420", "23401", "23410", "30412",
-	    "30421", "31402", "31420", "32401", "32410",
-	"04123", "04213", "14023", "14203", "24013", "24103", "04132",
-	    "04231", "14032", "14230", "24031", "24130",
+	"02314", "10324", "12304", "20314", "21304", "03124", "03214",
+	"13024", "13204", "23014", "23104", "30124", "30214", "31024",
+	"31204", "32014", "32104", "01243", "02143", "10243", "12043",
+	"20143", "21043", "01342", "02341", "10342", "12340", "20341",
+	"21340", "03142", "03241", "13042", "13240", "23041", "23140",
+	"30142", "30241", "31042", "31240", "32041", "32140", "01423",
+	"02413", "10423", "12403", "20413", "21403", "01432", "02431",
+	"10432", "12430", "20431", "21430", "03412", "03421", "13402",
+	"13420", "23401", "23410", "30412", "30421", "31402", "31420",
+	"32401", "32410", "04123", "04213", "14023", "14203", "24013",
+	"24103", "04132", "04231", "14032", "14230", "24031", "24130",
 	"04312", "04321", "14302", "14320", "24301", "24310", "34012",
-	    "34021", "34102", "34120", "34201", "34210",
-	"40123", "40213", "41023", "41203", "42013", "42103", "40132",
-	    "40231", "41032", "41230", "42031", "42130",
-	"40312", "40321", "41302", "41320", "42301", "42310", "43012",
-	    "43021", "43102", "43120", "43201", "43210", 0
+	"34021", "34102", "34120", "34201", "34210", "40123", "40213",
+	"41023", "41203", "42013", "42103", "40132", "40231", "41032",
+	"41230", "42031", "42130", "40312", "40321", "41302", "41320",
+	"42301", "42310", "43012", "43021", "43102", "43120", "43201",
+	"43210", 0
 };
 
 static const char *filter_names[8]
@@ -253,6 +209,14 @@ static const char *filter_names[8]
 	"SFinished"
 };
 
+static const char *filter_names_resume[]
+    = { "SHello",
+	"SChangeCipherSpec",
+	"SFinished",
+	"CChangeCipherSpec",
+	"CFinished"
+};
+
 static const char *filter_names_full[12]
     = { "SHello",
 	"SCertificate",
@@ -329,7 +293,9 @@ enum role role;
 
 int debug;
 int nonblock;
+int replay;
 int full;
+int resume;
 int timeout_seconds;
 int retransmit_milliseconds;
 int run_to_end;
@@ -343,8 +309,7 @@ int run_id;
 static void logfn(int level, const char *s)
 {
 	if (debug) {
-		fprintf(stdout, "%i %s|<%i> %s", run_id, role_name, level,
-			s);
+		fprintf(stdout, "%i %s|<%i> %s", run_id, role_name, level, s);
 	}
 }
 
@@ -358,8 +323,7 @@ static void auditfn(gnutls_session_t session, const char *s)
 static void drop(const char *packet)
 {
 	if (debug) {
-		fprintf(stdout, "%i %s| dropping %s\n", run_id, role_name,
-			packet);
+		log("dropping %s\n", packet);
 	}
 }
 
@@ -387,9 +351,7 @@ static void _process_error_or_timeout(int loc, int err, time_t tdiff)
 		if (err != GNUTLS_E_TIMEDOUT || tdiff >= 60) {
 			_process_error(loc, err, 0);
 		} else {
-			fprintf(stdout,
-				"%i %s| line %i: {spurious timeout} (fatal)",
-				run_id, role_name, loc);
+			log("line %i: {spurious timeout} (fatal)", loc);
 			exit(1);
 		}
 	}
@@ -399,8 +361,7 @@ static void _process_error_or_timeout(int loc, int err, time_t tdiff)
 
 static void rperror(const char *name)
 {
-	fprintf(stdout, "%i %s| %s: %s\n", run_id, role_name, name,
-		strerror(errno));
+	fprintf(stdout, "%i %s| %s\n", run_id, role_name, name);
 }
 
 // }}}
@@ -417,30 +378,34 @@ filter_packet_state_t state_packet_ClientKeyExchange = { 0 };
 filter_packet_state_t state_packet_ClientCertificateVerify = { 0 };
 filter_packet_state_t state_packet_ClientChangeCipherSpec = { 0 };
 filter_packet_state_t state_packet_ClientFinished = { 0 };
+filter_packet_state_t state_packet_ClientFinishedResume = { 0 };
 filter_packet_state_t state_packet_ServerChangeCipherSpec = { 0 };
 filter_packet_state_t state_packet_ServerFinished = { 0 };
+filter_packet_state_t state_packet_ServerFinishedResume = { 0 };
 
-filter_permute_state_t state_permute_ServerHello =
-    { {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}, 0, 0 };
-filter_permute_state_t state_permute_ServerHelloFull =
-    { {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}, 0, 0 };
-filter_permute_state_t state_permute_ServerFinished =
-    { {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}, 0, 0 };
-filter_permute_state_t state_permute_ClientFinished =
-    { {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}, 0, 0 };
-filter_permute_state_t state_permute_ClientFinishedFull =
-    { {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}, 0, 0 };
+static filter_permute_state_t state_permute_ServerHello =
+    { 0, {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}, 0, 0 };
+static filter_permute_state_t state_permute_ServerHelloFull =
+    { 0, {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}, 0, 0 };
+static filter_permute_state_t state_permute_ServerFinished =
+    { 0, {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}, 0, 0 };
+static filter_permute_state_t state_permute_ServerFinishedResume =
+    { 0, {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}, 0, 0 };
+static filter_permute_state_t state_permute_ClientFinished =
+    { 0, {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}, 0, 0 };
+static filter_permute_state_t state_permute_ClientFinishedResume =
+    { 0, {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}, 0, 0 };
+static filter_permute_state_t state_permute_ClientFinishedFull =
+    { 0, {{0, 0}, {0, 0}, {0, 0}, {0, 0}, {0, 0}}, 0, 0 };
 
 filter_fn filter_chain[32];
 int filter_current_idx;
 
-static void filter_permute_state_free_buffer(filter_permute_state_t *
-					     state)
+static void filter_permute_state_free_buffer(filter_permute_state_t * state)
 {
 	unsigned int i;
 
-	for (i = 0; i < sizeof(state->packets) / sizeof(state->packets[0]);
-	     i++) {
+	for (i = 0; i < sizeof(state->packets) / sizeof(state->packets[0]); i++) {
 		free(state->packets[i].data);
 		state->packets[i].data = NULL;
 	}
@@ -453,12 +418,12 @@ static void filter_clear_state(void)
 	filter_permute_state_free_buffer(&state_permute_ServerHello);
 	filter_permute_state_free_buffer(&state_permute_ServerHelloFull);
 	filter_permute_state_free_buffer(&state_permute_ServerFinished);
+	filter_permute_state_free_buffer(&state_permute_ServerFinishedResume);
 	filter_permute_state_free_buffer(&state_permute_ClientFinished);
-	filter_permute_state_free_buffer
-	    (&state_permute_ClientFinishedFull);
+	filter_permute_state_free_buffer(&state_permute_ClientFinishedResume);
+	filter_permute_state_free_buffer(&state_permute_ClientFinishedFull);
 
-	memset(&state_packet_ServerHello, 0,
-	       sizeof(state_packet_ServerHello));
+	memset(&state_packet_ServerHello, 0, sizeof(state_packet_ServerHello));
 	memset(&state_packet_ServerCertificate, 0,
 	       sizeof(state_packet_ServerCertificate));
 	memset(&state_packet_ServerKeyExchange, 0,
@@ -477,10 +442,14 @@ static void filter_clear_state(void)
 	       sizeof(state_packet_ClientChangeCipherSpec));
 	memset(&state_packet_ClientFinished, 0,
 	       sizeof(state_packet_ClientFinished));
+	memset(&state_packet_ClientFinishedResume, 0,
+	       sizeof(state_packet_ClientFinishedResume));
 	memset(&state_packet_ServerChangeCipherSpec, 0,
 	       sizeof(state_packet_ServerChangeCipherSpec));
 	memset(&state_packet_ServerFinished, 0,
 	       sizeof(state_packet_ServerFinished));
+	memset(&state_packet_ServerFinishedResume, 0,
+	       sizeof(state_packet_ServerFinishedResume));
 	memset(&state_permute_ServerHello, 0,
 	       sizeof(state_permute_ServerHello));
 	memset(&state_permute_ServerHelloFull, 0,
@@ -489,21 +458,49 @@ static void filter_clear_state(void)
 	       sizeof(state_permute_ServerFinished));
 	memset(&state_permute_ClientFinished, 0,
 	       sizeof(state_permute_ClientFinished));
+	memset(&state_permute_ClientFinishedResume, 0,
+	       sizeof(state_permute_ClientFinishedResume));
 	memset(&state_permute_ClientFinishedFull, 0,
 	       sizeof(state_permute_ClientFinishedFull));
+
+	state_permute_ServerHello.name = "ServerHello";
+	state_permute_ServerHelloFull.name = "ServerHelloFull";
+	state_permute_ServerFinished.name = "ServerFinished";
+	state_permute_ServerFinishedResume.name = "ServerFinishedResume";
+	state_permute_ClientFinished.name = "ClientFinished";
+	state_permute_ClientFinishedResume.name = "ClientFinishedResume";
+	state_permute_ClientFinishedFull.name = "ClientFinishedFull";
 }
 
+/* replay buffer */
+static int rbuffer[5 * 1024];
+unsigned rbuffer_size = 0;
+
 static void filter_run_next(gnutls_transport_ptr_t fd,
 			    const unsigned char *buffer, size_t len)
 {
+	int ret = 0;
 	filter_fn fn = filter_chain[filter_current_idx];
 	filter_current_idx++;
 	if (fn) {
 		fn(fd, buffer, len);
 	} else {
-		send((int) (intptr_t) fd, buffer, len, 0);
+		ret = send((int)(intptr_t) fd, buffer, len, 0);
 	}
 	filter_current_idx--;
+
+	if (ret > 0 && replay != 0) {
+		if (rbuffer_size == 0 && len < sizeof(rbuffer)) {
+			memcpy(rbuffer, buffer, len);
+			rbuffer_size = len;
+		} else if (rbuffer_size != 0) {
+			send((int)(intptr_t) fd, rbuffer, rbuffer_size, 0);
+			if (len < sizeof(rbuffer) && len > rbuffer_size) {
+				memcpy(rbuffer, buffer, len);
+				rbuffer_size = len;
+			}
+		}
+	}
 }
 
 // }}}
@@ -560,28 +557,24 @@ static int match_ClientCertificateVerify(const unsigned char *buffer,
 	    && buffer[13] == 15;
 }
 
-static int match_ClientChangeCipherSpec(const unsigned char *buffer,
-					size_t len)
+static int match_ClientChangeCipherSpec(const unsigned char *buffer, size_t len)
 {
 	return role == CLIENT && len >= 13 && buffer[0] == 20;
 }
 
 static int match_ClientFinished(const unsigned char *buffer, size_t len)
 {
-	return role == CLIENT && len >= 13 && buffer[0] == 22
-	    && buffer[4] == 1;
+	return role == CLIENT && len >= 13 && buffer[0] == 22 && buffer[4] == 1;
 }
 
-static int match_ServerChangeCipherSpec(const unsigned char *buffer,
-					size_t len)
+static int match_ServerChangeCipherSpec(const unsigned char *buffer, size_t len)
 {
 	return role == SERVER && len >= 13 && buffer[0] == 20;
 }
 
 static int match_ServerFinished(const unsigned char *buffer, size_t len)
 {
-	return role == SERVER && len >= 13 && buffer[0] == 22
-	    && buffer[4] == 1;
+	return role == SERVER && len >= 13 && buffer[0] == 22 && buffer[4] == 1;
 }
 
 // }}}
@@ -617,12 +610,14 @@ DECLARE_FILTER(ServerHello)
 static void filter_permute_state_run(filter_permute_state_t * state,
 				     int packetCount,
 				     gnutls_transport_ptr_t fd,
-				     const unsigned char *buffer,
-				     size_t len)
+				     const unsigned char *buffer, size_t len)
 {
 	unsigned char *data = malloc(len);
 	int packet = state->order[state->count];
 
+	if (debug > 2)
+		log("running permutation for %s/%d/%d\n", state->name, packetCount, state->count);
+
 	memcpy(data, buffer, len);
 	state->packets[packet].data = data;
 	state->packets[packet].size = len;
@@ -655,25 +650,39 @@ static void filter_permute_state_run(filter_permute_state_t * state,
 
 static match_fn permute_match_ServerHello[] =
     { match_ServerHello, match_ServerKeyExchange, match_ServerHelloDone };
+
 static match_fn permute_match_ServerHelloFull[] =
     { match_ServerHello, match_ServerCertificate, match_ServerKeyExchange,
 	match_ServerCertificateRequest, match_ServerHelloDone
 };
+
 static match_fn permute_match_ServerFinished[] =
     { match_ServerChangeCipherSpec, match_ServerFinished };
+
+static match_fn permute_match_ServerFinishedResume[] =
+    { match_ServerHello, match_ServerChangeCipherSpec, match_ServerFinished };
+
 static match_fn permute_match_ClientFinished[] =
     { match_ClientKeyExchange, match_ClientChangeCipherSpec,
-match_ClientFinished };
+	match_ClientFinished
+};
+
+static match_fn permute_match_ClientFinishedResume[] =
+    { match_ClientChangeCipherSpec, match_ClientFinished
+};
+
 static match_fn permute_match_ClientFinishedFull[] =
     { match_ClientCertificate, match_ClientKeyExchange,
 	match_ClientCertificateVerify, match_ClientChangeCipherSpec,
-	    match_ClientFinished
+	match_ClientFinished
 };
 
 DECLARE_PERMUTE(ServerHello)
     DECLARE_PERMUTE(ServerHelloFull)
+    DECLARE_PERMUTE(ServerFinishedResume)
     DECLARE_PERMUTE(ServerFinished)
     DECLARE_PERMUTE(ClientFinished)
+    DECLARE_PERMUTE(ClientFinishedResume)
     DECLARE_PERMUTE(ClientFinishedFull)
 // }}}
 // {{{ emergency deadlock resolution time bomb
@@ -709,7 +718,7 @@ gnutls_session_t session;
 static ssize_t writefn(gnutls_transport_ptr_t fd, const void *buffer,
 		       size_t len)
 {
-	filter_run_next(fd, (const unsigned char *) buffer, len);
+	filter_run_next(fd, (const unsigned char *)buffer, len);
 	return len;
 }
 
@@ -727,9 +736,8 @@ static void await(int fd, int timeout)
 
 static void cred_init(void)
 {
-	gnutls_datum_t key = { (unsigned char *) PUBKEY, sizeof(PUBKEY) };
-	gnutls_datum_t sec =
-	    { (unsigned char *) PRIVKEY, sizeof(PRIVKEY) };
+	gnutls_datum_t key = { (unsigned char *)PUBKEY, sizeof(PUBKEY) };
+	gnutls_datum_t sec = { (unsigned char *)PRIVKEY, sizeof(PRIVKEY) };
 
 	gnutls_certificate_allocate_credentials(&cred);
 
@@ -740,8 +748,7 @@ static void cred_init(void)
 static void session_init(int sock, int server)
 {
 	gnutls_init(&session,
-		    GNUTLS_DATAGRAM | (server ? GNUTLS_SERVER :
-				       GNUTLS_CLIENT)
+		    GNUTLS_DATAGRAM | (server ? GNUTLS_SERVER : GNUTLS_CLIENT)
 		    | GNUTLS_NONBLOCK * nonblock);
 	gnutls_priority_set_direct(session,
 				   "+CTYPE-OPENPGP:+CIPHER-ALL:+MAC-ALL:+ECDHE-RSA:+ANON-ECDH",
@@ -749,24 +756,21 @@ static void session_init(int sock, int server)
 	gnutls_transport_set_int(session, sock);
 
 	if (full) {
-		gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE,
-				       cred);
+		gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, cred);
 		if (server) {
 			gnutls_certificate_server_set_request(session,
 							      GNUTLS_CERT_REQUIRE);
 		}
 	} else if (server) {
-		gnutls_anon_server_credentials_t cred;
-		gnutls_anon_allocate_server_credentials(&cred);
-		gnutls_credentials_set(session, GNUTLS_CRD_ANON, cred);
+		gnutls_anon_server_credentials_t acred;
+		gnutls_anon_allocate_server_credentials(&acred);
+		gnutls_credentials_set(session, GNUTLS_CRD_ANON, acred);
 	} else {
-		gnutls_anon_client_credentials_t cred;
-		gnutls_anon_allocate_client_credentials(&cred);
-		gnutls_credentials_set(session, GNUTLS_CRD_ANON, cred);
+		gnutls_anon_client_credentials_t acred;
+		gnutls_anon_allocate_client_credentials(&acred);
+		gnutls_credentials_set(session, GNUTLS_CRD_ANON, acred);
 	}
 
-	gnutls_transport_set_push_function(session, writefn);
-
 	gnutls_dtls_set_mtu(session, 1400);
 	gnutls_dtls_set_timeouts(session, retransmit_milliseconds,
 				 timeout_seconds * 1000);
@@ -778,10 +782,41 @@ static void client(int sock)
 	time_t started = time(0);
 	const char *line = "foobar!";
 	char buffer[8192];
-	int len;
+	int len, ret;
+	gnutls_datum_t data = {NULL, 0};
 
 	session_init(sock, 0);
 
+	killtimer_set();
+
+	if (resume) {
+		do {
+			err = process_error(gnutls_handshake(session));
+			if (err != 0) {
+				int t = gnutls_dtls_get_timeout(session);
+				await(sock, t ? t : 100);
+			}
+		} while (err != 0);
+		process_error_or_timeout(err, time(0) - started);
+
+		ret = gnutls_session_get_data2(session, &data);
+		if (ret < 0) {
+			exit(1);
+		}
+		gnutls_deinit(session);
+
+		session_init(sock, 0);
+		gnutls_session_set_data(session, data.data, data.size);
+		gnutls_free(data.data);
+		data.data = NULL;
+
+		if (debug) {
+			fprintf(stdout, "%i %s| initial handshake complete\n", run_id, role_name);
+		}
+	}
+
+	gnutls_transport_set_push_function(session, writefn);
+
 	killtimer_set();
 	do {
 		err = process_error(gnutls_handshake(session));
@@ -792,26 +827,84 @@ static void client(int sock)
 	} while (err != 0);
 	process_error_or_timeout(err, time(0) - started);
 
-	killtimer_set();
-	die_on_error(gnutls_record_send(session, line, strlen(line)));
-
-	do {
-		await(sock, -1);
-		len =
-		    process_error(gnutls_record_recv
-				  (session, buffer, sizeof(buffer)));
-	} while (len < 0);
-
-	if (len > 0 && strncmp(line, buffer, len) == 0) {
-		exit(0);
-	} else {
-		exit(1);
+	if (debug) {
+		fprintf(stdout, "%i %s| handshake complete\n", run_id, role_name);
 	}
+
+	if (resume) {
+		killtimer_set();
+
+		do {
+			await(sock, -1);
+			len =
+			    process_error(gnutls_record_recv
+					  (session, buffer, sizeof(buffer)));
+		} while (len < 0);
+
+		log("received data\n");
+
+		die_on_error(gnutls_record_send(session, buffer, len));
+
+		log("sent data\n");
+		exit(0);
+
+	} else {
+		killtimer_set();
+		die_on_error(gnutls_record_send(session, line, strlen(line)));
+
+		log("sent data\n");
+
+		do {
+			await(sock, -1);
+			len =
+			    process_error(gnutls_record_recv
+				  (session, buffer, sizeof(buffer)));
+		} while (len < 0);
+
+		log("received data\n");
+
+		if (len > 0 && strncmp(line, buffer, len) == 0) {
+			exit(0);
+		} else {
+			exit(1);
+		}
+	}
+
+}
+
+static gnutls_datum_t saved_data = {NULL, 0};
+
+static gnutls_datum_t db_fetch(void *dbf, gnutls_datum_t key)
+{
+	gnutls_datum_t t = {NULL, 0};
+	t.data = malloc(saved_data.size);
+	if (t.data == NULL)
+		return t;
+	memcpy(t.data, saved_data.data, saved_data.size);
+	t.size = saved_data.size;
+
+	return t;
+}
+
+static int db_delete(void *dbf, gnutls_datum_t key)
+{
+	return 0;
+}
+
+static int db_store(void *dbf, gnutls_datum_t key, gnutls_datum_t data)
+{
+	saved_data.data = malloc(data.size);
+	if (saved_data.data == NULL)
+		return -1;
+	memcpy(saved_data.data, data.data, data.size);
+	saved_data.size = data.size;
+	return 0;
 }
 
 static void server(int sock)
 {
 	int err;
+	const char *line = "server foobar!";
 	time_t started = time(0);
 	char buffer[8192];
 	int len;
@@ -820,6 +913,39 @@ static void server(int sock)
 
 	await(sock, -1);
 
+	killtimer_set();
+	if (resume) {
+		gnutls_db_set_retrieve_function(session, db_fetch);
+		gnutls_db_set_store_function(session, db_store);
+		gnutls_db_set_remove_function(session, db_delete);
+		gnutls_db_set_ptr(session, NULL);
+
+		do {
+			err = process_error(gnutls_handshake(session));
+			if (err != 0) {
+				int t = gnutls_dtls_get_timeout(session);
+				await(sock, t ? t : 100);
+			}
+		} while (err != 0);
+		process_error_or_timeout(err, time(0) - started);
+
+		gnutls_deinit(session);
+
+		session_init(sock, 1);
+		gnutls_db_set_retrieve_function(session, db_fetch);
+		gnutls_db_set_store_function(session, db_store);
+		gnutls_db_set_remove_function(session, db_delete);
+		gnutls_db_set_ptr(session, NULL);
+
+		if (debug) {
+			fprintf(stdout, "%i %s| initial handshake complete\n", run_id, role_name);
+		}
+	}
+
+	gnutls_transport_set_push_function(session, writefn);
+
+	await(sock, -1);
+
 	killtimer_set();
 	do {
 		err = process_error(gnutls_handshake(session));
@@ -830,15 +956,50 @@ static void server(int sock)
 	} while (err != 0);
 	process_error_or_timeout(err, time(0) - started);
 
-	killtimer_set();
-	do {
-		await(sock, -1);
-		len =
-		    process_error(gnutls_record_recv
+	log("handshake complete\n");
+
+	if (resume) {
+		free(saved_data.data);
+		saved_data.data = NULL;
+	}
+
+	if (resume) {
+		killtimer_set();
+		die_on_error(gnutls_record_send(session, line, strlen(line)));
+
+		log("sent data\n");
+
+		do {
+			await(sock, -1);
+			len =
+			    process_error(gnutls_record_recv
 				  (session, buffer, sizeof(buffer)));
-	} while (len < 0);
+		} while (len < 0);
+
+		log("received data\n");
+
+		if (len > 0 && strncmp(line, buffer, len) == 0) {
+			exit(0);
+		} else {
+			exit(1);
+		}
+	} else {
+		killtimer_set();
+
+		do {
+			await(sock, -1);
+			len =
+			    process_error(gnutls_record_recv
+					  (session, buffer, sizeof(buffer)));
+		} while (len < 0);
+
+		log("received data\n");
+
+		die_on_error(gnutls_record_send(session, buffer, len));
+
+		log("sent data\n");
+	}
 
-	die_on_error(gnutls_record_send(session, buffer, len));
 	exit(0);
 }
 
@@ -857,11 +1018,11 @@ static void udp_sockpair(int *socks)
 	socks[0] = socket(AF_INET6, SOCK_DGRAM, 0);
 	socks[1] = socket(AF_INET6, SOCK_DGRAM, 0);
 
-	bind(socks[0], (struct sockaddr *) &sa, sizeof(sa));
-	bind(socks[1], (struct sockaddr *) &sb, sizeof(sb));
+	bind(socks[0], (struct sockaddr *)&sa, sizeof(sa));
+	bind(socks[1], (struct sockaddr *)&sb, sizeof(sb));
 
-	connect(socks[1], (struct sockaddr *) &sa, sizeof(sa));
-	connect(socks[0], (struct sockaddr *) &sb, sizeof(sb));
+	connect(socks[1], (struct sockaddr *)&sa, sizeof(sa));
+	connect(socks[0], (struct sockaddr *)&sb, sizeof(sb));
 }
 #endif
 
@@ -877,8 +1038,8 @@ static int run_test(void)
 	}
 
 	if (nonblock) {
-		fcntl(fds[0], F_SETFL, (long) O_NONBLOCK);
-		fcntl(fds[1], F_SETFL, (long) O_NONBLOCK);
+		fcntl(fds[0], F_SETFL, (long)O_NONBLOCK);
+		fcntl(fds[1], F_SETFL, (long)O_NONBLOCK);
 	}
 
 	if (!(pid1 = fork())) {
@@ -895,9 +1056,9 @@ static int run_test(void)
 		rperror("fork client");
 		exit(2);
 	}
-	while (waitpid(pid2, &status2, 0) < 0 && errno == EINTR);
+	while (waitpid(pid2, &status2, 0) < 0 && errno == EINTR) ;
 	kill(pid1, 15);
-	while (waitpid(pid1, 0, 0) < 0 && errno == EINTR);
+	while (waitpid(pid1, 0, 0) < 0 && errno == EINTR) ;
 
 	close(fds[0]);
 	close(fds[1]);
@@ -909,7 +1070,7 @@ static int run_test(void)
 	}
 }
 
-static filter_fn filters[8]
+static filter_fn filters[]
     = { filter_packet_ServerHello,
 	filter_packet_ServerKeyExchange,
 	filter_packet_ServerHelloDone,
@@ -920,7 +1081,15 @@ static filter_fn filters[8]
 	filter_packet_ServerFinished
 };
 
-static filter_fn filters_full[12]
+static filter_fn filters_resume[]
+    = { filter_packet_ServerHello,
+	filter_packet_ServerChangeCipherSpec,
+	filter_packet_ServerFinished,
+	filter_packet_ClientChangeCipherSpec,
+	filter_packet_ClientFinished
+};
+
+static filter_fn filters_full[]
     = { filter_packet_ServerHello,
 	filter_packet_ServerCertificate,
 	filter_packet_ServerKeyExchange,
@@ -940,16 +1109,39 @@ static int run_one_test(int dropMode, int serverFinishedPermute,
 {
 	int fnIdx = 0;
 	int res, filterIdx;
-	filter_fn *local_filters = full ? filters_full : filters;
-	const char **local_filter_names =
-	    full ? filter_names_full : filter_names;
-	const char **permutation_namesX =
-	    full ? permutation_names5 : permutation_names3;
-	int filter_count = full ? 12 : 8;
+	filter_fn *local_filters;
+	const char **local_filter_names;
+	const char **client_finished_permutation_names;
+	const char **server_finished_permutation_names;
+	const char **server_hello_permutation_names;
+	int filter_count;
+
+	if (full) {
+		local_filters = filters_full;
+		local_filter_names = filter_names_full;
+		filter_count = sizeof(filters_full)/sizeof(filters_full[0]);
+		client_finished_permutation_names = permutation_names5;
+		server_finished_permutation_names = permutation_names2;
+		server_hello_permutation_names = permutation_names5;
+	} else if (resume) {
+		local_filters = filters_resume;
+		local_filter_names = filter_names_resume;
+		filter_count = sizeof(filters_resume)/sizeof(filters_resume[0]);
+		client_finished_permutation_names = permutation_names2;
+		server_finished_permutation_names = permutation_names3;
+		server_hello_permutation_names = NULL;
+	} else {
+		local_filters = filters;
+		local_filter_names = filter_names;
+		filter_count = sizeof(filters)/sizeof(filters[0]);
+		client_finished_permutation_names = permutation_names3;
+		server_finished_permutation_names = permutation_names2;
+		server_hello_permutation_names = permutation_names3;
+	}
+
 	run_id =
 	    ((dropMode * 2 + serverFinishedPermute) * (full ? 120 : 6) +
-	     serverHelloPermute) * (full ? 120 : 6) +
-	    clientFinishedPermute;
+	     serverHelloPermute) * (full ? 120 : 6) + clientFinishedPermute;
 
 	filter_clear_state();
 
@@ -961,6 +1153,18 @@ static int run_one_test(int dropMode, int serverFinishedPermute,
 		filter_chain[fnIdx++] = filter_permute_ClientFinishedFull;
 		state_permute_ClientFinishedFull.order =
 		    permutations5[clientFinishedPermute];
+
+		filter_chain[fnIdx++] = filter_permute_ServerFinished;
+		state_permute_ServerFinished.order =
+		    permutations2[serverFinishedPermute];
+	} else if (resume) {
+		filter_chain[fnIdx++] = filter_permute_ServerFinishedResume;
+		state_permute_ServerFinishedResume.order =
+		    permutations3[serverFinishedPermute];
+
+		filter_chain[fnIdx++] = filter_permute_ClientFinishedResume;
+		state_permute_ClientFinishedResume.order =
+		    permutations2[clientFinishedPermute];
 	} else {
 		filter_chain[fnIdx++] = filter_permute_ServerHello;
 		state_permute_ServerHello.order =
@@ -969,11 +1173,11 @@ static int run_one_test(int dropMode, int serverFinishedPermute,
 		filter_chain[fnIdx++] = filter_permute_ClientFinished;
 		state_permute_ClientFinished.order =
 		    permutations3[clientFinishedPermute];
-	}
 
-	filter_chain[fnIdx++] = filter_permute_ServerFinished;
-	state_permute_ServerFinished.order =
-	    permutations2[serverFinishedPermute];
+		filter_chain[fnIdx++] = filter_permute_ServerFinished;
+		state_permute_ServerFinished.order =
+		    permutations2[serverFinishedPermute];
+	}
 
 	if (dropMode) {
 		for (filterIdx = 0; filterIdx < filter_count; filterIdx++) {
@@ -1002,12 +1206,12 @@ static int run_one_test(int dropMode, int serverFinishedPermute,
 		break;
 	}
 
-	fprintf(stdout, "SHello(%s), ",
-		permutation_namesX[serverHelloPermute]);
+	if (!resume)
+		fprintf(stdout, "SHello(%s), ", server_hello_permutation_names[serverHelloPermute]);
 	fprintf(stdout, "SFinished(%s), ",
-		permutation_names2[serverFinishedPermute]);
+		server_finished_permutation_names[serverFinishedPermute]);
 	fprintf(stdout, "CFinished(%s) :- ",
-		permutation_namesX[clientFinishedPermute]);
+		client_finished_permutation_names[clientFinishedPermute]);
 	if (dropMode) {
 		for (filterIdx = 0; filterIdx < filter_count; filterIdx++) {
 			if (dropMode & (1 << filterIdx)) {
@@ -1164,8 +1368,7 @@ static int run_all_tests(int childcount)
 					} else {
 						register_child(pid);
 						result |=
-						    wait_children
-						    (childcount);
+						    wait_children(childcount);
 					}
 				}
 
@@ -1197,9 +1400,11 @@ int main(int argc, const char *argv[])
 	int serverHelloPermute = 0;
 	int clientFinishedPermute = 0;
 	int batch = 0;
+	unsigned single = 0;
 	int arg;
 
 	nonblock = 0;
+	replay = 0;
 	debug = 0;
 	timeout_seconds = 120;
 	retransmit_milliseconds = 100;
@@ -1227,15 +1432,22 @@ int main(int argc, const char *argv[])
 			batch = 1;
 		} else if (strcmp("-d", argv[arg]) == 0) {
 			char *end;
-			int level = strtol(argv[arg + 1], &end, 10);
-			if (*end == '\0') {
-				debug = level;
-				arg++;
+			int level;
+
+			if (arg+1 < argc) {
+				level = strtol(argv[arg + 1], &end, 10);
+				if (*end == '\0') {
+					debug = level;
+					arg++;
+				} else
+					debug++;
 			} else {
 				debug++;
 			}
 		} else if (strcmp("-nb", argv[arg]) == 0) {
 			nonblock = 1;
+		} else if (strcmp("-r", argv[arg]) == 0) {
+			replay = 1;
 		} else if (strcmp("-timeout", argv[arg]) == 0) {
 			char *end;
 			int val;
@@ -1270,8 +1482,25 @@ int main(int argc, const char *argv[])
 				FAIL_ARG(j);
 			}
 		} else if (strcmp("-full", argv[arg]) == 0) {
+			if (resume) {
+				fprintf(stderr, "You cannot combine full with resume\n");
+				exit(1);
+			}
+
 			full = 1;
+		} else if (strcmp("-resume", argv[arg]) == 0) {
+			if (full) {
+				fprintf(stderr, "You cannot combine full with resume\n");
+				exit(1);
+			}
+
+			resume = 1;
 		} else if (strcmp("-shello", argv[arg]) == 0) {
+			if (resume) {
+				fprintf(stderr, "Please use -sfinished instead of -shello\n");
+				exit(1);
+			}
+
 			NEXT_ARG(shello);
 			if (!parse_permutation
 			    (argv[arg],
@@ -1279,26 +1508,46 @@ int main(int argc, const char *argv[])
 			     permutation_names3, &serverHelloPermute)) {
 				FAIL_ARG(shell);
 			}
+			single++;
 		} else if (strcmp("-sfinished", argv[arg]) == 0) {
-			NEXT_ARG(sfinished);
-			if (!parse_permutation
-			    (argv[arg], permutation_names2,
-			     &serverFinishedPermute)) {
-				FAIL_ARG(sfinished);
-			}
-		} else if (strcmp("-cfinished", argv[arg]) == 0) {
+			const char **pname;
 			NEXT_ARG(cfinished);
+			if (resume) pname = permutation_names3;
+			else pname = permutation_names2;
 			if (!parse_permutation
-			    (argv[arg],
-			     full ? permutation_names5 :
-			     permutation_names3, &clientFinishedPermute)) {
+			    (argv[arg], pname,
+			     &serverFinishedPermute)) {
 				FAIL_ARG(cfinished);
 			}
+			single++;
+		} else if (strcmp("-cfinished", argv[arg]) == 0) {
+			const char **pname;
+			NEXT_ARG(cfinished);
+			if (full) pname = permutation_names5;
+			else if (resume) pname = permutation_names2;
+			else pname = permutation_names3;
+			if (!parse_permutation
+			    (argv[arg], pname,
+			     &clientFinishedPermute)) {
+				FAIL_ARG(cfinished);
+			}
+			single++;
 		} else {
 			int drop;
-			int filter_count = full ? 12 : 8;
-			const char **local_filter_names =
-			    full ? filter_names_full : filter_names;
+			int filter_count;
+			const char **local_filter_names;
+
+			if (full) {
+				local_filter_names = filter_names_full;
+				filter_count = sizeof(filters_full)/sizeof(filters_full[0]);
+			} else if (resume) {
+				local_filter_names = filter_names_resume;
+				filter_count = sizeof(filters_resume)/sizeof(filters_resume[0]);
+			} else {
+				local_filter_names = filter_names;
+				filter_count = sizeof(filters)/sizeof(filters[0]);
+			}
+
 			for (drop = 0; drop < filter_count; drop++) {
 				if (strcmp
 				    (local_filter_names[drop],
@@ -1312,6 +1561,7 @@ int main(int argc, const char *argv[])
 					argv[arg]);
 				exit(8);
 			}
+			single++;
 		}
 	}
 
@@ -1322,12 +1572,20 @@ int main(int argc, const char *argv[])
 	gnutls_global_set_audit_log_function(auditfn);
 	gnutls_global_set_log_level(debug);
 
-	if (dropMode || serverFinishedPermute || serverHelloPermute
-	    || clientFinishedPermute) {
+	if (single) {
+		if (debug)
+			fprintf(stderr, "single test mode\n");
 		return run_one_test(dropMode, serverFinishedPermute,
-				    serverHelloPermute,
-				    clientFinishedPermute);
+				    serverHelloPermute, clientFinishedPermute);
 	} else {
+		if (debug)
+			fprintf(stderr, "multi test mode\n");
+
+		if (resume) {
+			fprintf(stderr, "full run not implemented yet for resumed runs\n");
+			exit(5);
+		}
+
 		job_pids = calloc(sizeof(int), job_limit);
 		if (batch) {
 			return run_tests_from_id_list(job_limit);
