@@ -112,9 +112,10 @@ const gnutls_datum_t server_key = { server_key_pem,
 
 #define MAX_BUF 24*1024
 
-static void client(int fd, const char *prio)
+static void client(int fd, const char *prio, int ign)
 {
 	int ret;
+	unsigned i;
 	char buffer[MAX_BUF + 1];
 	const char* err;
 	gnutls_anon_client_credentials_t anoncred;
@@ -172,6 +173,44 @@ static void client(int fd, const char *prio)
 			gnutls_protocol_get_name
 			(gnutls_protocol_get_version(session)));
 
+	/* Test sending */
+	for (i = 1; i < 16384; i++) {
+		do {
+			ret = gnutls_record_send(session, buffer, i);
+		} while (ret == GNUTLS_E_AGAIN
+			 || ret == GNUTLS_E_INTERRUPTED);
+
+		if (ret < 0) {
+			fail("server (%s): Error sending %d byte packet: %s\n", prio, i, gnutls_strerror(ret));
+			terminate();
+		}
+	}
+
+	/* Try sending a bit more */
+	i = 21056;
+	do {
+		ret = gnutls_record_send(session, buffer, i);
+	} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+
+	if (ret < 0) {
+		fail("server (%s): Error sending %d byte packet: %s\n",
+		     prio, i, gnutls_strerror(ret));
+		exit(1);
+	} else if (ign == 0 && ret != 16384) {
+		fail("server (%s): Error sending %d byte packet; sent %d bytes instead of 16384\n", prio, i, ret);
+		exit(1);
+	}
+	
+	ret = gnutls_alert_send(session, GNUTLS_AL_WARNING, GNUTLS_A_USER_CANCELED);
+	if (ret < 0) {
+		fail("server (%s): Error sending alert\n", prio);
+		exit(1);
+	}
+
+	/* make sure we are not blocked forever */
+	gnutls_record_set_timeout(session, 10000);
+
+	/* Test receiving */
 	do {
 		do {
 			ret = gnutls_record_recv(session, buffer, MAX_BUF);
@@ -277,6 +316,30 @@ static void server(int fd, const char *prio, int ign)
 			gnutls_protocol_get_name
 			(gnutls_protocol_get_version(session)));
 
+	/* Here we do both a receive and a send test because if valgrind
+	 * detects an error on the peer, the main process will never know.
+	 */
+
+	/* Test receiving */
+	do {
+		do {
+			ret = gnutls_record_recv(session, buffer, MAX_BUF);
+		} while (ret == GNUTLS_E_AGAIN
+			 || ret == GNUTLS_E_INTERRUPTED);
+	} while (ret > 0);
+	
+	if (ret != GNUTLS_E_WARNING_ALERT_RECEIVED ||
+		gnutls_alert_get(session) != GNUTLS_A_USER_CANCELED) {
+
+		if (ret <= 0) {
+			if (ret != 0) {
+				fail("client: Error: %s\n", gnutls_strerror(ret));
+				exit(1);
+			}
+		}
+	}
+
+	/* Test sending */
 	for (i = 1; i < 16384; i++) {
 		do {
 			ret = gnutls_record_send(session, buffer, i);
@@ -342,10 +405,9 @@ static void start(const char *prio, int ign)
 		/* parent */
 		close(fd[1]);
 		server(fd[0], prio, ign);
-		kill(child, SIGTERM);
 	} else {
 		close(fd[0]);
-		client(fd[1], prio);
+		client(fd[1], prio, ign);
 		exit(0);
 	}
 }
@@ -353,33 +415,24 @@ static void start(const char *prio, int ign)
 #define AES_CBC "NONE:+VERS-TLS1.0:-CIPHER-ALL:+AES-128-CBC:+SHA1:+SIGN-ALL:+COMP-NULL:+ANON-ECDH:+CURVE-ALL"
 #define AES_CBC_SHA256 "NONE:+VERS-TLS1.2:-CIPHER-ALL:+RSA:+AES-128-CBC:+AES-256-CBC:+SHA256:+SIGN-ALL:+COMP-NULL:+ANON-ECDH:+CURVE-ALL"
 #define AES_GCM "NONE:+VERS-TLS1.2:-CIPHER-ALL:+RSA:+AES-128-GCM:+MAC-ALL:+SIGN-ALL:+COMP-NULL:+ANON-ECDH:+CURVE-ALL"
+#define AES_CCM "NONE:+VERS-TLS1.2:-CIPHER-ALL:+RSA:+AES-128-CCM:+MAC-ALL:+SIGN-ALL:+COMP-NULL:+ANON-ECDH:+CURVE-ALL"
+#define AES_CCM_8 "NONE:+VERS-TLS1.2:-CIPHER-ALL:+RSA:+AES-128-CCM-8:+MAC-ALL:+SIGN-ALL:+COMP-NULL:+ANON-ECDH:+CURVE-ALL"
 
 #define ARCFOUR_SHA1 "NONE:+VERS-TLS1.0:-CIPHER-ALL:+ARCFOUR-128:+SHA1:+SIGN-ALL:+COMP-NULL:+ANON-ECDH:+CURVE-ALL"
 #define ARCFOUR_MD5 "NONE:+VERS-TLS1.0:-CIPHER-ALL:+ARCFOUR-128:+MD5:+SIGN-ALL:+COMP-NULL:+ANON-ECDH:+CURVE-ALL:+RSA"
 
 #define NULL_SHA1 "NONE:+VERS-TLS1.0:-CIPHER-ALL:+NULL:+SHA1:+SIGN-ALL:+COMP-NULL:+ANON-ECDH:+RSA:+CURVE-ALL"
 
-#define NEW_AES_CBC "NONE:+VERS-TLS1.0:-CIPHER-ALL:+AES-128-CBC:+SHA1:+SIGN-ALL:+COMP-NULL:+ANON-ECDH:+CURVE-ALL:%NEW_PADDING"
-#define NEW_ARCFOUR_SHA1 "NONE:+VERS-TLS1.0:-CIPHER-ALL:+ARCFOUR-128:+SHA1:+SIGN-ALL:+COMP-NULL:+ANON-ECDH:+CURVE-ALL:%NEW_PADDING"
-#define NEW_AES_CBC_SHA256 "NONE:+VERS-TLS1.2:-CIPHER-ALL:+RSA:+AES-128-CBC:+AES-256-CBC:+SHA256:+SIGN-ALL:+COMP-NULL:+ANON-ECDH:+CURVE-ALL:%NEW_PADDING"
-#define NEW_AES_GCM "NONE:+VERS-TLS1.2:-CIPHER-ALL:+RSA:+AES-128-GCM:+MAC-ALL:+SIGN-ALL:+COMP-NULL:+ANON-ECDH:+CURVE-ALL:%NEW_PADDING"
-
 #define ARCFOUR_SHA1_ZLIB "NONE:+VERS-TLS1.0:-CIPHER-ALL:+ARCFOUR-128:+SHA1:+SIGN-ALL:+COMP-DEFLATE:+ANON-ECDH:+CURVE-ALL"
-#define NEW_ARCFOUR_SHA1_ZLIB "NONE:+VERS-TLS1.0:-CIPHER-ALL:+ARCFOUR-128:+SHA1:+SIGN-ALL:+COMP-DEFLATE:+ANON-ECDH:+CURVE-ALL:%NEW_PADDING"
+
+#define AES_GCM_ZLIB "NONE:+VERS-TLS1.2:-CIPHER-ALL:+AES-128-GCM:+AEAD:+SIGN-ALL:+COMP-DEFLATE:+RSA:+CURVE-ALL"
+#define CHACHA_POLY1305 "NONE:+VERS-TLS1.2:-CIPHER-ALL:+RSA:+CHACHA20-POLY1305:+MAC-ALL:+SIGN-ALL:+COMP-ALL:+ECDHE-RSA:+CURVE-ALL"
 
 static void ch_handler(int sig)
 {
 	int status;
 	wait(&status);
-	if (WEXITSTATUS(status) != 0 ||
-	    (WIFSIGNALED(status) && WTERMSIG(status) == SIGSEGV)) {
-		if (WIFSIGNALED(status))
-			fail("Child died with sigsegv\n");
-		else
-			fail("Child died with status %d\n",
-			     WEXITSTATUS(status));
-		terminate();
-	}
+	check_wait_status(status);
 	return;
 }
 
@@ -387,23 +440,26 @@ void doit(void)
 {
 	signal(SIGCHLD, ch_handler);
 
-	start(NULL_SHA1, 0);
-
-	start(NEW_ARCFOUR_SHA1, 1);
-	start(NEW_AES_CBC, 1);
-	start(NEW_AES_CBC_SHA256, 1);
-	start(NEW_AES_GCM, 1);
-
 	start(AES_CBC, 1);
 	start(AES_CBC_SHA256, 1);
 	start(AES_GCM, 0);
+	start(AES_CCM, 0);
+	start(AES_CCM_8, 0);
+
+#ifndef ENABLE_FIPS140
+	start(NULL_SHA1, 0);
 
 	start(ARCFOUR_SHA1, 0);
 	start(ARCFOUR_MD5, 0);
+	start(CHACHA_POLY1305, 0);
+
+# ifdef HAVE_LIBZ
+	start(ARCFOUR_SHA1_ZLIB, 0);
+# endif
+#endif
 
 #ifdef HAVE_LIBZ
-	start(ARCFOUR_SHA1_ZLIB, 0);
-	start(NEW_ARCFOUR_SHA1_ZLIB, 1);
+	start(AES_GCM_ZLIB, 0);
 #endif
 }
 
