@@ -16,7 +16,8 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with GnuTLS.  If not, see <https://www.gnu.org/licenses/>.
+ * along with GnuTLS; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,7 +50,8 @@ static void try(void)
 	/* Server stuff. */
 	gnutls_certificate_credentials_t serverx509cred;
 	gnutls_dh_params_t dh_params;
-	const gnutls_datum_t p3 = { (unsigned char *)pkcs3, strlen(pkcs3) };
+	const gnutls_datum_t p3 =
+	    { (unsigned char *) pkcs3, strlen(pkcs3) };
 	gnutls_session_t server;
 	int sret = GNUTLS_E_AGAIN;
 	/* Client stuff. */
@@ -65,8 +67,9 @@ static void try(void)
 
 	/* Init server */
 	gnutls_certificate_allocate_credentials(&serverx509cred);
-	gnutls_certificate_set_x509_key_mem(serverx509cred, &server_cert,
-					    &server_key, GNUTLS_X509_FMT_PEM);
+	gnutls_certificate_set_x509_key_mem(serverx509cred,
+					    &server_cert, &server_key,
+					    GNUTLS_X509_FMT_PEM);
 
 	gnutls_dh_params_init(&dh_params);
 	gnutls_dh_params_import_pkcs3(dh_params, &p3, GNUTLS_X509_FMT_PEM);
@@ -74,9 +77,11 @@ static void try(void)
 
 	gnutls_init(&server, GNUTLS_SERVER);
 	gnutls_certificate_server_set_request(server, GNUTLS_CERT_REQUEST);
-	gnutls_credentials_set(server, GNUTLS_CRD_CERTIFICATE, serverx509cred);
+	gnutls_credentials_set(server, GNUTLS_CRD_CERTIFICATE,
+				serverx509cred);
 
-	gnutls_priority_set_direct(server, "NORMAL:-VERS-ALL:+VERS-TLS1.2",
+	gnutls_priority_set_direct(server,
+				   "NORMAL",
 				   NULL);
 	gnutls_transport_set_push_function(server, server_push);
 	gnutls_transport_set_pull_function(server, server_pull);
@@ -88,13 +93,11 @@ static void try(void)
 	if (ret < 0)
 		exit(1);
 
-	ret = gnutls_certificate_set_x509_key_mem(
-		clientx509cred, &cli_cert, &cli_key, GNUTLS_X509_FMT_PEM);
+	ret = gnutls_certificate_set_x509_key_mem(clientx509cred, &cli_cert, &cli_key, GNUTLS_X509_FMT_PEM);
 	if (ret < 0)
 		exit(1);
 
-	ret = gnutls_certificate_set_x509_trust_mem(clientx509cred, &ca_cert,
-						    GNUTLS_X509_FMT_PEM);
+	ret = gnutls_certificate_set_x509_trust_mem(clientx509cred, &ca_cert, GNUTLS_X509_FMT_PEM);
 	if (ret < 0)
 		exit(1);
 
@@ -102,14 +105,11 @@ static void try(void)
 	if (ret < 0)
 		exit(1);
 
-	ret = gnutls_certificate_set_x509_key_mem(clientx509cred2,
-						  &server2_cert, &server2_key,
-						  GNUTLS_X509_FMT_PEM);
+	ret = gnutls_certificate_set_x509_key_mem(clientx509cred2, &server2_cert, &server2_key, GNUTLS_X509_FMT_PEM);
 	if (ret < 0)
 		exit(1);
 
-	ret = gnutls_certificate_set_x509_trust_mem(clientx509cred2, &ca_cert,
-						    GNUTLS_X509_FMT_PEM);
+	ret = gnutls_certificate_set_x509_trust_mem(clientx509cred2, &ca_cert, GNUTLS_X509_FMT_PEM);
 	if (ret < 0)
 		exit(1);
 
@@ -118,12 +118,11 @@ static void try(void)
 		exit(1);
 
 	ret = gnutls_credentials_set(client, GNUTLS_CRD_CERTIFICATE,
-				     clientx509cred);
+				clientx509cred);
 	if (ret < 0)
 		exit(1);
 
-	ret = gnutls_priority_set_direct(
-		client, "NORMAL:-VERS-ALL:+VERS-TLS1.2:-KX-ALL:+RSA", NULL);
+	ret = gnutls_priority_set_direct(client, "NORMAL:-KX-ALL:+RSA", NULL);
 	if (ret < 0)
 		exit(1);
 
@@ -134,16 +133,15 @@ static void try(void)
 	HANDSHAKE(client, server);
 
 	if (gnutls_kx_get(client) != GNUTLS_KX_RSA) {
-		fail("got unexpected key exchange algorithm: %s (expected RSA)\n",
-		     gnutls_kx_get_name(gnutls_kx_get(client)));
+		fail("got unexpected key exchange algorithm: %s (expected RSA)\n", gnutls_kx_get_name(gnutls_kx_get(client)));
 		exit(1);
 	}
 
 	/* switch server's certificate and rehandshake */
-	gnutls_credentials_set(client, GNUTLS_CRD_CERTIFICATE, clientx509cred2);
+	gnutls_credentials_set(client, GNUTLS_CRD_CERTIFICATE,
+				clientx509cred2);
 
-	HANDSHAKE_EXPECT(client, server, GNUTLS_E_AGAIN,
-			 GNUTLS_E_SESSION_USER_ID_CHANGED);
+	HANDSHAKE_EXPECT(client, server, GNUTLS_E_AGAIN, GNUTLS_E_SESSION_USER_ID_CHANGED);
 
 	gnutls_deinit(client);
 	gnutls_deinit(server);
