@@ -44,7 +44,6 @@ int main()
 #include <arpa/inet.h>
 #include <unistd.h>
 #include <signal.h>
-#include <assert.h>
 #include <gnutls/gnutls.h>
 #include <gnutls/dtls.h>
 
@@ -65,7 +64,7 @@ static void client_log_func(int level, const char *str)
 
 #define MAX_BUF 1024
 
-static void client(int fd, const char *prio)
+static void client(int fd)
 {
 	int ret;
 	char buffer[MAX_BUF + 1];
@@ -84,9 +83,12 @@ static void client(int fd, const char *prio)
 	/* Initialize TLS session
 	 */
 	gnutls_init(&session, GNUTLS_CLIENT);
-	gnutls_handshake_set_timeout(session, get_timeout());
+	gnutls_handshake_set_timeout(session, 20 * 1000);
 
-	assert(gnutls_priority_set_direct(session, prio, NULL) >= 0);
+	/* Use default priorities */
+	gnutls_priority_set_direct(session,
+				   "NORMAL:-KX-ALL:+ECDHE-RSA",
+				   NULL);
 
 	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
 
@@ -156,7 +158,7 @@ static void terminate(void)
 	exit(1);
 }
 
-static void server(int fd, const char *prio)
+static void server(int fd)
 {
 	int ret;
 	gnutls_certificate_credentials_t xcred;
@@ -187,9 +189,14 @@ static void server(int fd, const char *prio)
 	}
 
 	gnutls_init(&session, GNUTLS_SERVER);
-	gnutls_handshake_set_timeout(session, get_timeout());
+	gnutls_handshake_set_timeout(session, 20 * 1000);
 
-	assert(gnutls_priority_set_direct(session, prio, NULL) >= 0);
+	/* avoid calling all the priority functions, since the defaults
+	 * are adequate.
+	 */
+	gnutls_priority_set_direct(session,
+				   "NORMAL:-KX-ALL:+ECDHE-RSA",
+				   NULL);
 
 	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
 
@@ -246,13 +253,11 @@ static void server(int fd, const char *prio)
 		success("server: finished\n");
 }
 
-static 
-void run(const char *name, const char *prio)
+void doit(void)
 {
 	int fd[2];
 	int ret;
 
-	success("trying: %s\n", name);
 	signal(SIGPIPE, SIG_IGN);
 
 	ret = socketpair(AF_UNIX, SOCK_STREAM, 0, fd);
@@ -273,22 +278,15 @@ void run(const char *name, const char *prio)
 		/* parent */
 
 		close(fd[0]);
-		client(fd[1], prio);
+		client(fd[1]);
 
 		wait(&status);
 		check_wait_status(status);
 	} else {
 		close(fd[1]);
-		server(fd[0], prio);
+		server(fd[0]);
 		exit(0);
 	}
 }
 
-void doit(void)
-{
-	run("tls1.2", "NORMAL:-KX-ALL:+ECDHE-RSA:-VERS-ALL:+VERS-TLS1.2");
-	run("tls1.3", "NORMAL:-VERS-ALL:+VERS-TLS1.3");
-	run("default", "NORMAL");
-}
-
 #endif				/* _WIN32 */
